/*
 * Title        :  Chart.java
 * Package      :  test
 * Project      :  BioGrid2
 * Description	:  		
 * Created on   :  Dec 4, 2006
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.chart;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

public class Chart {

    private Vector chartTypes;
    private Vector columnLabels;
    private Hashtable dataSet;
    private String liveUpdateURL;
    private String extra;
    
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        // TODO Auto-generated method stub
        Chart chart = new Chart();
        chart.setColumnLabels(new String[] {null, "2001", "2002", "2003", "2004"});

        // Region A
        chart.addChartType("line");
        Vector a = new Vector();
        a.add("5");
        a.add("10");
        a.add("30");
        a.add("63");
        chart.setRowData("Region A", a);
        
        // Region B
        chart.addChartType("column");
        Vector b = new Vector();
        b.add("40");
        b.add("20");
        b.add("65");
        b.add("90");
        chart.setRowData("Region B", b);

        // Region C
        chart.addChartType("column");
        Vector c = new Vector();
        c.add("56");
        c.add("35");
        c.add("40");
        c.add("90");
        chart.setRowData("Region C", c);
        
        System.out.println(chart.toString());
    }

    public Chart() {
        this.chartTypes = new Vector();
        this.columnLabels = new Vector();
        this.dataSet = new Hashtable();
    }
    
    /**
     * @return the columnLabels
     */
    public Vector getColumnLabels() {
        return columnLabels;
    }

    public String getColumnLabel(int index) throws Exception {
        return (String) columnLabels.get(index);
    }
    
    /**
     * @param columnLabels the columnLabels to set
     */
    public void setColumnLabels(Vector columnLabels) {
        this.columnLabels = columnLabels;
    }
    
    public void setColumnLabels(String[] columnLabels) {
        Vector labels = new Vector();
        for (int i = 0; i < columnLabels.length; i++) {
            labels.add(columnLabels[i]);
        }
        this.columnLabels = labels;
    }
    
    public void addColumnLabel(String columnLabel) {
        this.columnLabels.add(columnLabel);
    }
    
    public void setColumnLabel(int index, String columnLabel) throws Exception {
        this.columnLabels.set(index, columnLabel);
    }

    /**
     * @return the chartTypes
     */
    public Vector getChartTypes() {
        return chartTypes;
    }

    public String getChartType(int index) throws Exception {
        return (String) columnLabels.get(index);
    }
    
    /**
     * @param chartTypes the chartTypes to set
     */
    public void setChartTypes(Vector chartTypes) {
        this.chartTypes = chartTypes;
    }

    public void setChartTypes(String[] chartTypes) {
        Vector types = new Vector();
        for (int i = 0; i < chartTypes.length; i++) {
            types.add(chartTypes[i]);
        }
        this.chartTypes = types;
    }
    
    public void addChartType(String chartType) {
        this.chartTypes.add(chartType);
    }
    
    public void setChartType(int index, String chartType) throws Exception {
        this.chartTypes.set(index, chartType);
    }

    /**
     * @return the rowLabels
     */
    public Set getRowLabels() {
        return dataSet.keySet();
    }

    /**
     * @return the rowData
     */
    public Hashtable getRowData() {
        return this.dataSet;
    }

    public Vector getRowData(String rowLabel) throws Exception {
        return (Vector) this.dataSet.get(rowLabel);
    }
    
    public void setRowData(String rowLabel, Vector rowData) {
        this.dataSet.put(rowLabel, rowData);
    }

    public void addRowData(String rowLabel, String value) {
        Vector v = (Vector) this.dataSet.get(rowLabel);
        if (v == null) {
            v = new Vector();
        }
        v.add(value);
        this.dataSet.put(rowLabel,v);
    }
    
    public String toString() {

        String chartXML = "";
        chartXML += "<chart>\n";
        chartXML += toString_chart_type();
        chartXML += toString_chart_data();
        chartXML += toString_live_update();
        chartXML += toString_extra();
        chartXML += "</chart>\n";
        return chartXML;
    }
    
    private String toString_chart_type() {
        String chartTypeStart = "<chart_type>";
        String chartTypeEnd = "</chart_type>\n";
        String typeXML = "";
        
        typeXML += chartTypeStart;
        for (Iterator it = chartTypes.iterator(); it.hasNext();) {
            if (chartTypes.size() == 1) {
                typeXML += it.next();    
            } else {
                typeXML += "\n"+toString_label(it.next());                
            }
        }
        typeXML += chartTypeEnd;
        
        return typeXML;
    }
    
    private String toString_chart_data() {
        String dataXML = "";

        dataXML += "<chart_data>\n";
        dataXML += toString_column_labels();
        dataXML += toString_row_data();
        dataXML += "</chart_data>\n";
        return dataXML;
    }

    private String toString_column_labels() {
        String rowStart = "<row>\n";
        String rowEnd = "</row>\n";
        String headers = "";
        headers += rowStart + toString_label(null);
        for (Iterator it = columnLabels.iterator(); it.hasNext();) {
            headers += toString_label(it.next());
        }
        headers += rowEnd;
        return headers;
    }

    private String toString_row_data() {
        String rowStart = "<row>\n";
        String rowEnd = "</row>\n";
        String allRows = "";
        String row = "";
        String label = "";
        // for each key, write out key and values.
        for (Iterator it = dataSet.keySet().iterator(); it.hasNext();) {
            row = rowStart;
            label = (String) it.next();
            row += toString_label(label);
            Vector data = (Vector) dataSet.get(label);
            for (Iterator dit = data.iterator(); dit.hasNext();) {
                row += toString_element(dit.next());
            }
            row += rowEnd;
            allRows = row + allRows;
        }
        return allRows;
    }

    private String toString_element(Object value) {
        String numberStart = "<number>";
        String numberEnd = "</number>\n";
        String nullValue = "<null/>\n";
        String xml = "";
        
        if (value != null) {
            xml = (numberStart+value+numberEnd);
        } else {
            xml = nullValue;
        }
        return xml;
    }
    
    private String toString_label(Object value) {
        String stringStart = "<string>";
        String stringEnd = "</string>\n";
        String nullValue = "<null/>\n";
        String xml = "";
        
        if (value != null) {
            xml = (stringStart+value+stringEnd);
        } else {
            xml = nullValue;
        }
        return xml;
    }
    
    private String toString_live_update() {
        String xml = "";
        xml += "<live_update url=\'"+liveUpdateURL+"\' delay=\'5\' />\n";
        return xml;
    }

    /**
     * @return the liveUpdateURL
     */
    public String getLiveUpdateURL() {
        return liveUpdateURL;
    }

    /**
     * @param liveUpdateURL the liveUpdateURL to set
     */
    public void setLiveUpdateURL(String liveUpdateURL) {
        this.liveUpdateURL = liveUpdateURL;
    }

    /**
     * @return the extra
     */
    public String getExtra() {
        return extra;
    }

    /**
     * @param extra the extra to set
     */
    public void setExtra(String extra) {
        this.extra = extra;
    }
    
    private String toString_extra() {
        return this.extra+"\n";
    }
}
