/*
 * Title        :  DBQuerier.java
 * Package      :  org.gridbus.biogrid.data
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  Apr 28, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.data;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Collection;
import java.util.Vector;

import org.apache.commons.fileupload.FileItem;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.biogrid.pdb.GridUser;
import org.gridbus.biogrid.pdb.Project;
import org.gridbus.biogrid.pdb.ProjectFile;
import org.gridbus.biogrid.pdb.Resource;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;
import org.ietf.jgss.GSSCredential;


/**
 * @author hag
 *
 */
public class DBQuerier {

	private DBConnection dbc=null;
	private Connection conn=null;
	
	//INPUT queries
	private String insertInput = "INSERT INTO input(filename,data,size,experiment_id) VALUES(?,?,?,?)";
	private String selectInputs = "SELECT input_id,filename,data,size FROM input";
	private String selectInput = selectInputs + " WHERE experiment_id=?";
	private String selectInputsDetails = "SELECT input_id,filename,size FROM input";
	private String selectInputDetails = selectInputsDetails + " WHERE experiment_id=?";
	private String deleteInput = "DELETE FROM input WHERE input_id=?";

    //USERS
    private String selectUsers = "SELECT user_id,identity FROM user";
    private String selectUser = selectUsers + " WHERE identity=?";
    private String selectUserByIdentityAndId = selectUser + " AND user_id=?";
    private String selectUserById = selectUsers + " WHERE user_id=?";
    private String selectUserCredential = "SELECT credential FROM user WHERE user_id=?";
    private String updateUserCredential = "UPDATE user SET credential=? WHERE user_id=?";
	//USER DATA RESOURCES
    private String insertUserDataResource = "INSERT INTO user_dataresource(user_id,dataresource_id) VALUES(?,?)";
    private String deleteUserDataResources = "DELETE FROM user_dataresource WHERE user_id=?";
    private String deleteUserDataResource = deleteUserDataResources + " AND dataresource_id=?";
    private String setDefaultUserDataResource = "UPDATE user_dataresource SET dflt=? WHERE user_id=? AND dataresource_id=?";
    private String removeDefaultUserDataResource = "UPDATE user_dataresource SET dflt=? WHERE user_id=?";
    private String selectDefaultUserDataResources = "SELECT dataresource_id FROM user_dataresource WHERE user_id=? AND dflt=?";
    //USER RESOURCES
    private String insertUserResource = "INSERT INTO user_resource(user_id,resource_id) VALUES(?,?)";
	private String deleteUserResources = "DELETE FROM user_resource WHERE user_id=?";
	private String deleteUserResource = deleteUserResources + " AND resource_id=?";
    //USER PROJECTS
	private String insertUserProject = "INSERT INTO user_experiment(user_id,experiment_id) VALUES (?,?)";
	private String deleteUserProjects = "DELETE FROM user_experiment WHERE user_id=?";
	private String deleteUserProject = deleteUserProjects + " AND experiment_id=?";
	private String selectUserProjects = "SELECT user_id,experiment_id FROM user_experiment WHERE user_id=?";
	private String selectUserProject = selectUserProjects + " AND experiment_id=?";
	private String selectUserProjectsOrdered = selectUserProjects + " ORDER BY experiment_id DESC";

	//RESOURCES
	private String insertResource = "INSERT INTO resource(hostname,contact,queue,type,price,joblimit) VALUES(?,?,?,?,?,?)";
    private String selectResources = "SELECT resource.resource_id,hostname,contact,queue,type,price,joblimit FROM resource,user_resource WHERE user_id=? and resource.resource_id=user_resource.resource_id";
	private String selectResource = "SELECT resource_id,hostname,contact,queue,type,price,joblimit FROM resource WHERE resource_id=?";
	private String deleteResource = "DELETE FROM resource WHERE resource_id=?";
    private String deleteResourceProjects = "DELETE FROM experiment_resource WHERE resource_id=?";
    private String selectResourceDataResource = "SELECT dataresource.dataresource_id,hostname,homedir FROM dataresource,resource_dataresource WHERE dataresource.dataresource_id=resource_dataresource.dataresource_id AND resource_dataresource.resource_id=?";
    private String deleteResourceDataResource = "DELETE FROM resource_dataresource WHERE resource_id=?";
    private String deleteDataResourceLinks = "DELETE FROM resource_dataresource WHERE dataresource_id=?";
    private String setResourceDataResource = "INSERT INTO resource_dataresource(resource_id,dataresource_id) VALUES(?,?)";
    
    //DATA RESOURCES
    private String insertDataResource = "INSERT INTO dataresource(hostname,homedir) VALUES(?,?)";
    private String selectDataResources = "SELECT dataresource.dataresource_id,hostname,homedir FROM dataresource,user_dataresource WHERE user_id=? and dataresource.dataresource_id=user_dataresource.dataresource_id";
    private String selectDataResource = "SELECT dataresource_id,hostname,homedir FROM dataresource WHERE dataresource_id=?";
    private String deleteDataResource = "DELETE FROM dataresource WHERE dataresource_id=?";
    
	//PROJECT
	private String insertProject = "INSERT INTO experiment(name,created_on) VALUES(?,?)";
	private String deleteProjectResults = "DELETE FROM result WHERE experiment_id=?";
	private String insertProjectResource = "INSERT INTO experiment_resource(experiment_id,resource_id) VALUES(?,?)";
	private String deleteProjectResources = "DELETE FROM experiment_resource WHERE experiment_id=?";
	private String deleteProjectApplications = "DELETE FROM experiment_application WHERE experiment_id=?";
	private String selectProjectApplication = "SELECT experiment_id,application_id FROM experiment_application WHERE experiment_id=?";	
	private String selectProjects = "SELECT experiment_id,name,created_on,application_id FROM experiment";
	private String selectProjectResources = "SELECT experiment_id,resource_id FROM experiment_resource WHERE experiment_id=?";
    private String selectProject = selectProjects + " WHERE experiment_id=?";
	private String selectProjectsPage = selectProjects + " WHERE experiment_id > ?";
	private String selectProjectsPageOrdered = selectProjects + " WHERE experiment_id <= ? ORDER BY experiment_id DESC";
	private String selectProjectsPageMax = "SELECT MAX(experiment_id) AS max_id FROM experiment";
	private String findUserProject = "SELECT experiment.experiment_id FROM experiment,user_experiment WHERE experiment.experiment_id=user_experiment.experiment_id AND user_experiment.user_id=? AND experiment.name=?";
    private String insertProjectDataResource = "INSERT INTO experiment_dataresource(experiment_id,dataresource_id) VALUES (?,?)";
    private String deleteProjectDataResources = "DELETE FROM experiment_dataresource WHERE experiment_id=?";
    private String selectProjectDataResources = "SELECT experiment_id,dataresource_id FROM experiment_dataresource WHERE experiment_id=?";
    
	//APPLICATION queries
	private String selectApplication = "SELECT application_id,name,data FROM application WHERE application_id=?";

	//QoS queries
	private String selectDeadline = "SELECT deadline FROM qos WHERE experiment_id=?";
	private String selectBudget = "SELECT budget FROM qos WHERE experiment_id=?";
	private String selectOptimisation = "SELECT optimisation FROM qos WHERE experiment_id=?";
	private String deleteQoS = "DELETE FROM QoS WHERE experiment_id=?";
	private String insertQoS = "INSERT INTO QoS(experiment_id,deadline,budget,optimisation) VALUES(?,?,?,?)";
	private String checkQoS = "SELECT * FROM qos WHERE experiment_id=?";
	
	/**
	 * @throws Exception
	 */
	public DBQuerier() throws Exception {
		this.dbc = new DBConnection();
		connect();
	}
	/**
	 * @param dbc
	 * @throws Exception
	 * 
	 */
	public DBQuerier(DBConnection dbc) throws Exception {
		this.dbc = dbc;
		connect();
	}

	/**
	 * Constructor specifying the JDBC driver class name to use
	 * @param driver
	 * @param connectionString
	 * @param username
	 * @param password
	 * @throws Exception
	 */
	public DBQuerier(String driver,String connectionString,String username,String password) throws Exception{
        dbc = new DBConnection();
        dbc.setDriver(driver);
        dbc.setConnectionString(connectionString);
        dbc.setUsername(username);
        dbc.setPassword(password);
		connect();
	}

	/**
	 * Sets up the database connection
	 * @throws Exception
	 */
	public void connect() throws Exception{
		conn = dbc.getConnection();
	}

	/**
	 * Closes the database connection
	 * @throws Exception
	 */
	public void disconnect() throws Exception{
		dbc.closeConnection();
	}
	
	/**
	 * @param projectId
	 * @throws Exception
	 */
	public void removeProjectResults(int projectId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteProjectResults);
		ps.setInt(1,projectId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param file
	 * @param experimentId
	 * @throws Exception
	 */
	public void addInput(File file,int experimentId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(insertInput);
		ps.setString(1,file.getName());
		ps.setBinaryStream(2,new FileInputStream(file),Integer.parseInt(""+file.length()));
		ps.setLong(3,file.length());
		ps.setInt(4,experimentId);
		ps.execute();
		ps.close();
	}

	/**
	 * @param file
	 * @param experimentId
	 * @throws Exception
	 */
	public void addInput(FileItem file,int experimentId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(insertInput);
		ps.setString(1,file.getName());
		ps.setBinaryStream(2,file.getInputStream(),Integer.parseInt(""+file.getSize()));
		ps.setLong(3,file.getSize());
		ps.setInt(4,experimentId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param inputId
	 * @throws Exception
	 */
	public void removeInput(int inputId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteInput);
		ps.setInt(1,inputId);
		ps.execute();
		ps.close();
	}

	/**
	 * @param name
	 * @return
	 * @throws Exception
	 */
	public int addProject(String name) throws Exception {
		int projectId = -1;
		PreparedStatement ps = conn.prepareStatement(insertProject, Statement.RETURN_GENERATED_KEYS);
		ps.setString(1,name);
		ps.setLong(2,System.currentTimeMillis());
		ps.execute();
		ResultSet rs = ps.getGeneratedKeys();
		rs.next();
		projectId = rs.getInt(1);
		rs.close();
		ps.close();
		return projectId;
	}

	/**
	 * @param name
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public int findProject(String name, String userId) throws Exception {
		int projectId = -1;
		
		PreparedStatement ps = conn.prepareStatement(findUserProject);
		ps.setString(1,userId);
		ps.setString(2,name);
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			projectId = rs.getInt("experiment_id");
		}
		rs.close();
		ps.close();

		return projectId;
	}

	/**
	 * @param experimentId
	 * @return
	 * @throws Exception
	 */
	public Collection getInput(int experimentId) throws Exception {
		Blob b = null;
		ProjectFile input = null;
		Vector inputs = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectInput);
		ps.setInt(1,experimentId);
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			input = new ProjectFile();
			b = rs.getBlob("data");
			input.setName(rs.getString("filename"));
			input.setId(rs.getInt("input_id"));
			input.setSize(rs.getLong("size"));
			input.setData(b.getBytes(1,Integer.parseInt(""+b.length())));
			inputs.addElement(input);
		}
		rs.close();
		ps.close();
		return inputs;
	}

    // FIXME : Are the getInputDetails methods required??
    
	/**
	 * @param experimentId
	 * @return
	 * @throws Exception
	 */
	public Collection getInputDetails(int experimentId) throws Exception {
		Blob b = null;
		ProjectFile input = null;
		Vector inputs = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectInputDetails);
		ps.setInt(1,experimentId);
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			input = new ProjectFile();
			input.setName(rs.getString("filename"));
			input.setId(rs.getInt("input_id"));
			input.setSize(rs.getLong("size"));
			inputs.addElement(input);
		}
		rs.close();
		ps.close();
		return inputs;
	}

	/**
	 * @param pageSize
	 * @param pageNum
	 * @return
	 * @throws Exception
	 */
	public Collection getProjectsPage(int pageSize,int pageNum) throws Exception {
		Project project = null;
		Vector projects = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectProjectsPage);
		ps.setFetchSize(pageSize);
		ps.setMaxRows(pageSize);
		
		ps.setInt(1,pageSize*(pageNum-1));
		
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			project = new Project(rs.getInt("experiment_id"),rs.getString("name"),rs.getLong("created_on"),rs.getString("application_id"));
			projects.add(project);
		}
		rs.close();
		ps.close();
		return projects;
	}

	/**
	 * @param pageSize
	 * @param pageNum
	 * @param max
	 * @return
	 * @throws Exception
	 */
	public Collection getProjectsPageOrdered(int pageSize,int pageNum,int max) throws Exception {
		Project project = null;
		Vector projects = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectProjectsPageOrdered);
		ps.setFetchSize(pageSize);
		ps.setMaxRows(pageSize);
		
		int range = max-(pageSize*(pageNum-1));
		
		System.out.println(selectProjectsPageOrdered);
		
		ps.setInt(1,range);
		
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			project = new Project(rs.getInt("experiment_id"),rs.getString("name"),rs.getLong("created_on"),rs.getString("application_id"));
			projects.add(project);
		}
		rs.close();
		ps.close();
		return projects;
	}
	
	/**
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public int getProjectsPageMax(String userId) throws Exception {
		int max = 0;
		PreparedStatement ps = conn.prepareStatement(selectProjectsPageMax);
		ps.setString(1,userId);
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			max = rs.getInt("max_id");
		}
		rs.close();
		ps.close();
		return max;
	}

	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public Project getProject(int projectId) throws Exception {
		Project project = null;
        
        if (projectId < 0) {
            project = new Project();
            project.setId(projectId);
            return project;
        }
        
		PreparedStatement ps = conn.prepareStatement(selectProject);
		ps.setInt(1,projectId);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			project = new Project(projectId,rs.getString("name"),rs.getLong("created_on"),rs.getString("application_id"));
		}
		rs.close();
		ps.close();
		return project;
	}

	
	
	/**
	 * @param identity
	 * @return
	 * @throws Exception
	 */
	public GridUser getUser(String identity) throws Exception {
		GridUser user = null;
		PreparedStatement ps = conn.prepareStatement(selectUser);
		ps.setString(1,identity);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			user = new GridUser(rs.getString("user_id"),rs.getString("identity"));
		}
		rs.close();
		ps.close();
		return user;
	}

	/**
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public boolean userExists(String userId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(selectUserById);
		ps.setString(1,userId);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			return true;
		}
		rs.close();
		ps.close();
		return false;
	}
	
	/**
	 * @param identity
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public GridUser getUser(String identity, String userId) throws Exception {
		GridUser user = null;
		PreparedStatement ps = conn.prepareStatement(selectUserByIdentityAndId);
		ps.setString(1,identity);
		ps.setString(2,userId);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			user = new GridUser(rs.getString("user_id"),rs.getString("identity"));
		}
		rs.close();
		ps.close();
		return user;
	}

	
	/**
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public GSSCredential getUserCredential(String userId) throws Exception {
		ExtendedGSSCredential credential = null;
		ExtendedGSSManager manager = null;
		
		PreparedStatement ps = conn.prepareStatement(selectUserCredential);
		ps.setString(1,userId);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			byte[] b = rs.getBytes("credential");
			manager = (ExtendedGSSManager)ExtendedGSSManager.getInstance();
			try {
				credential = (ExtendedGSSCredential) manager.createCredential(b,
		                        ExtendedGSSCredential.IMPEXP_OPAQUE,
		                        GSSCredential.DEFAULT_LIFETIME,
		                        null, // use default mechanism - GSI
		                        GSSCredential.INITIATE_AND_ACCEPT);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		rs.close();
		ps.close();
		return credential;
	}

	/**
	 * @param userId
	 * @param credential
	 * @throws Exception
	 */
	public void setUserCredential(String userId, ExtendedGSSCredential credential) throws Exception {
		try {
			PreparedStatement ps = conn.prepareStatement(updateUserCredential);
			byte[] b = credential.export(ExtendedGSSCredential.IMPEXP_OPAQUE,null);
			ps.setBytes(1,b);
			ps.setString(2,userId);
			ps.execute();
			ps.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @param userId
	 * @param projectId
	 * @throws Exception
	 */
	public void addUserProject(String userId,int projectId) throws Exception {
		if (userId != null) {
			PreparedStatement ps = conn.prepareStatement(insertUserProject);
			ps.setString(1,userId);
			ps.setInt(2,projectId);
			ps.execute();
			ps.close();
		}
	}

	/**
	 * @param userId
	 * @throws Exception
	 */
	public void removeUserProjects(String userId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteUserProjects);
		ps.setString(1,userId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param userId
	 * @param projectId
	 * @throws Exception
	 */
	public void removeUserProject(String userId,int projectId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteUserProject);
		ps.setString(1,userId);
		ps.setInt(2,projectId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param userId
	 * @return
	 * @throws Exception
	 */
	public Collection getUserProjects(String userId) throws Exception {
		Project project = null;
		Vector projects = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectUserProjectsOrdered);
		ps.setString(1,userId);
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			project = getProject(rs.getInt("experiment_id"));
			projects.add(project);
		}
		rs.close();
		ps.close();
		return projects;
	}

	/**
	 * @param userId
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public Project getUserProject(String userId,int projectId) throws Exception {
		Project project = null;
		PreparedStatement ps = conn.prepareStatement(selectUserProject);
		ps.setString(1,userId);
		ps.setInt(2,projectId);
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			project = getProject(rs.getInt("experiment_id"));
		}
		rs.close();
		ps.close();
		return project;
	}




	/**
	 * @param projectId
	 * @throws Exception
	 */
	public void removeProjectApplications(int projectId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteProjectApplications);
		ps.setInt(1,projectId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public ProjectFile getProjectApplication(int projectId) throws Exception {
		ProjectFile application = null;
		PreparedStatement ps = conn.prepareStatement(selectProjectApplication);
		ps.setInt(1,projectId);
		
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			application = getApplication(rs.getInt("application_id"));
		}
		rs.close();
		ps.close();
		return application;
	}

    
    // START RESOURCES //

    /**
     * @param resourceId
     * @return
     * @throws Exception
     */
    public Resource getResource(int resourceId) throws Exception {
        Resource resource = null;
        PreparedStatement ps = conn.prepareStatement(selectResource);
        ps.setInt(1,resourceId);
        ResultSet rs = ps.executeQuery();
        if (rs.next()){
            resource = new Resource(rs.getString("hostname"),rs.getString("contact"),rs.getInt("type"),rs.getFloat("price"),rs.getInt("resource_id"),rs.getString("queue"),rs.getInt("joblimit"));
            resource.setDataResource(getResourceDataResource(resourceId));
        }
        rs.close();
        ps.close();
        return resource;
    }
    
	/**
	 * @param userId 
	 * @return resources
	 * @throws Exception
	 */
	public Collection getResources(String userId) throws Exception {
		Resource resource = null;
		Vector resources = new Vector();
		PreparedStatement ps = conn.prepareStatement(selectResources);
        ps.setString(1,userId);
		ResultSet rs = ps.executeQuery();
		while (rs.next()){
			resource = new Resource(rs.getString("hostname"),rs.getString("contact"),rs.getInt("type"),rs.getFloat("price"),rs.getInt("resource_id"),rs.getString("queue"),rs.getInt("joblimit"));
            resource.setDataResource(getResourceDataResource(rs.getInt("resource_id")));
			resources.add(resource);
		}
		rs.close();
		ps.close();
		return resources;
	}

    /**
     * @param userId 
     * @param hostname 
     * @param contact 
     * @param queue 
     * @param type 
     * @param price 
     * @param dataresourceId 
     * @param joblimit 
     * @throws Exception
     */
    public void addResource(String userId, String hostname, String contact, String queue, int type, float price, int dataresourceId, int joblimit) throws Exception {
        int resourceId = -1;
        PreparedStatement ps = conn.prepareStatement(insertResource, Statement.RETURN_GENERATED_KEYS);
        ps.setString(1,hostname);
        ps.setString(2,contact);
        ps.setString(3,queue);
        ps.setInt(4,type);
        ps.setFloat(5,price);
        ps.setInt(6,joblimit);
        ps.execute();
        ResultSet rs = ps.getGeneratedKeys();
        rs.next();
        resourceId = rs.getInt(1);
        rs.close();
        ps.close();

        setUserResource(userId,resourceId);
        if (dataresourceId > 0) {
            setResourceDataResource(resourceId,dataresourceId);
        }
    }
    
    /**
     * @param userId 
     * @param resourceId
     * @throws Exception
     */
    public void removeResource(String userId, int resourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteResource);
        ps.setInt(1,resourceId);
        ps.execute();
        ps.close();

        removeUserResource(userId,resourceId);
        removeResourceProjects(resourceId);
        removeResourceDataResource(resourceId);
    }

    /**
     * @param userId 
     * @param projectId
     * @param resourceId
     * @throws Exception
     */
    public void setUserResource(String userId, int resourceId) throws Exception {
        if (resourceId >= 0) {
            PreparedStatement ps = conn.prepareStatement(insertUserResource, ResultSet.TYPE_FORWARD_ONLY, 
                    ResultSet.CONCUR_UPDATABLE, ResultSet.CLOSE_CURSORS_AT_COMMIT);
            ps.setString(1,userId);
            ps.setInt(2,resourceId);
            ps.execute();
            ps.close();
        }
    }

    /**
     * @param userId 
     * @param resourceId 
     * @param projectId
     * @throws Exception
     */
    public void removeUserResource(String userId, int resourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteUserResource);
        ps.setString(1,userId);
        ps.setInt(2,resourceId);
        ps.execute();
        ps.close();
    }
    
    /**
     * @param projectId
     * @param resourceId
     * @throws Exception
     */
    public void setProjectResource(int projectId, int resourceId) throws Exception {
        if (resourceId >= 0) {
            PreparedStatement ps = conn.prepareStatement(insertProjectResource, ResultSet.TYPE_FORWARD_ONLY, 
                    ResultSet.CONCUR_UPDATABLE, ResultSet.CLOSE_CURSORS_AT_COMMIT);
            ps.setInt(1,projectId);
            ps.setInt(2,resourceId);
            ps.execute();
            ps.close();
        }
    }

    /**
     * @param projectId
     * @throws Exception
     */
    public void removeProjectResources(int projectId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteProjectResources);
        ps.setInt(1,projectId);
        ps.execute();
        ps.close();
    }
    
    /**
     * @param projectId
     * @return
     * @throws Exception
     */
    public Collection getProjectResources(int projectId) throws Exception {
        Resource resource = null;
        Vector resources = new Vector();
        PreparedStatement ps = conn.prepareStatement(selectProjectResources);
        ps.setInt(1,projectId);
        ResultSet rs = ps.executeQuery();
        while (rs.next()){
            resource = getResource(rs.getInt("resource_id"));
            resources.add(resource);
        }
        rs.close();
        ps.close();
        return resources;
    }

    /**
     * @param resourceId
     * @throws Exception
     */
    public void removeResourceProjects(int resourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteResourceProjects);
        ps.setInt(1,resourceId);
        ps.execute();
        ps.close();
    }
    
    // END RESOURCES //


    // START DATA RESOURCES //

    /**
     * @param dataresourceId
     * @return
     * @throws Exception
     */
    public DataResource getDataResource(int dataresourceId) throws Exception {
        DataResource dataresource = null;
        PreparedStatement ps = conn.prepareStatement(selectDataResource);
        ps.setInt(1,dataresourceId);
        ResultSet rs = ps.executeQuery();
        if (rs.next()){
            dataresource = new DataResource(rs.getInt("dataresource_id"),rs.getString("hostname"),rs.getString("homedir"));
        }
        rs.close();
        ps.close();
        return dataresource;
    }
    
    /**
     * @param userId 
     * @return dataresources
     * @throws Exception
     */
    public Collection getDataResources(String userId) throws Exception {
        DataResource dataresource = null;
        Vector dataresources = new Vector();
        PreparedStatement ps = conn.prepareStatement(selectDataResources);
        ps.setString(1,userId);
        ResultSet rs = ps.executeQuery();
        while (rs.next()){
            dataresource = new DataResource(rs.getInt("dataresource_id"),rs.getString("hostname"),rs.getString("homedir"));
            dataresources.add(dataresource);
        }
        rs.close();
        ps.close();
        return dataresources;
    }

    /**
     * @param userId 
     * @param hostname 
     * @param homedir 
     * @throws Exception
     */
    public void addDataResource(String userId, String hostname, String homedir) throws Exception {
        int dataresourceId = -1;
        PreparedStatement ps = conn.prepareStatement(insertDataResource, Statement.RETURN_GENERATED_KEYS);
        ps.setString(1,hostname);
        ps.setString(2,homedir);
        ps.execute();
        ResultSet rs = ps.getGeneratedKeys();
        rs.next();
        dataresourceId = rs.getInt(1);
        rs.close();
        ps.close();

        setUserDataResource(userId,dataresourceId);
    }
    
    /**
     * @param userId 
     * @param dataresourceId
     * @throws Exception
     */
    public void removeDataResource(String userId, int dataresourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteDataResource);
        ps.setInt(1,dataresourceId);
        ps.execute();
        ps.close();

        removeUserDataResource(userId,dataresourceId);
        removeDataResourceLinks(dataresourceId);
    }

    /**
     * @param userId 
     * @param dataresourceId 
     * @throws Exception
     */
    public void setUserDataResource(String userId, int dataresourceId) throws Exception {
        if (dataresourceId >= 0) {
            PreparedStatement ps = conn.prepareStatement(insertUserDataResource, ResultSet.TYPE_FORWARD_ONLY, 
                    ResultSet.CONCUR_UPDATABLE, ResultSet.CLOSE_CURSORS_AT_COMMIT);
            ps.setString(1,userId);
            ps.setInt(2,dataresourceId);
            ps.execute();
            ps.close();
        }
    }

    /**
     * @param userId 
     * @param dataresourceId 
     * @throws Exception
     */
    public void removeUserDataResource(String userId, int dataresourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteUserDataResource);
        ps.setString(1,userId);
        ps.setInt(2,dataresourceId);
        ps.execute();
        ps.close();
    }

    /**
     * @param resourceId 
     * @param dataresourceId 
     * @throws Exception
     */
    public void setResourceDataResource(int resourceId, int dataresourceId) throws Exception {
        if (dataresourceId >= 0) {
            PreparedStatement ps = conn.prepareStatement(setResourceDataResource, ResultSet.TYPE_FORWARD_ONLY, 
                    ResultSet.CONCUR_UPDATABLE, ResultSet.CLOSE_CURSORS_AT_COMMIT);
            ps.setInt(1,resourceId);
            ps.setInt(2,dataresourceId);
            ps.execute();
            ps.close();
        }
    }

    /**
     * @param resourceId 
     * @throws Exception
     */
    public void removeResourceDataResource(int resourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteResourceDataResource);
        ps.setInt(1,resourceId);
        ps.execute();
        ps.close();
    }

    /**
     * @param dataresourceId 
     * @throws Exception
     */
    public void removeDataResourceLinks(int dataresourceId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteDataResourceLinks);
        ps.setInt(1,dataresourceId);
        ps.execute();
        ps.close();
    }    

    /**
     * @param resourceId
     * @return
     * @throws Exception
     */
    public DataResource getResourceDataResource(int resourceId) throws Exception {
        DataResource dataresource = null;
        PreparedStatement ps = conn.prepareStatement(selectResourceDataResource);
        ps.setInt(1,resourceId);
        ResultSet rs = ps.executeQuery();
        if (rs.next()){
            dataresource = new DataResource(rs.getInt("dataresource_id"),rs.getString("hostname"),rs.getString("homedir"));
        }
        rs.close();
        ps.close();
        return dataresource;
    }
    
    /**
     * @param projectId
     * @param dataresourceId
     * @throws Exception
     */
    public void setProjectDataResource(int projectId, int dataresourceId) throws Exception {
        if (dataresourceId >= 0) {
            PreparedStatement ps = conn.prepareStatement(insertProjectDataResource, ResultSet.TYPE_FORWARD_ONLY, 
                    ResultSet.CONCUR_UPDATABLE, ResultSet.CLOSE_CURSORS_AT_COMMIT);
            ps.setInt(1,projectId);
            ps.setInt(2,dataresourceId);
            ps.execute();
            ps.close();
        }
    }
    
    public void setDefaultUserDataResource(String userId, int dataresourceId) throws Exception {
        removeDefaultUserDataResource(userId);
        PreparedStatement ps = conn.prepareStatement(setDefaultUserDataResource);
        ps.setBoolean(1,true);
        ps.setString(2,userId);
        ps.setInt(3,dataresourceId);
        ps.execute();
        ps.close();
    }

    public void removeDefaultUserDataResource(String userId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(removeDefaultUserDataResource);
        ps.setBoolean(1,false);
        ps.setString(2,userId);
        ps.execute();
        ps.close();
    }
    
    /**
     * @param projectId
     * @throws Exception
     */
    public void removeProjectDataResources(int projectId) throws Exception {
        PreparedStatement ps = conn.prepareStatement(deleteProjectDataResources);
        ps.setInt(1,projectId);
        ps.execute();
        ps.close();
    }
    
    /**
     * @param projectId
     * @return
     * @throws Exception
     */
    public DataResource getProjectDataResource(int projectId) throws Exception {
        DataResource dataresource = null;
        PreparedStatement ps = conn.prepareStatement(selectProjectDataResources);
        ps.setInt(1,projectId);
        ResultSet rs = ps.executeQuery();
        if (rs.next()){
            dataresource = getDataResource(rs.getInt("dataresource_id"));
        }
        rs.close();
        ps.close();
        return dataresource;
    }

    /**
     * @param projectId
     * @return
     * @throws Exception
     */
    public DataResource getDefaultUserDataResource(String userId) throws Exception {
        DataResource dataresource = null;
        PreparedStatement ps = conn.prepareStatement(selectDefaultUserDataResources);
        ps.setString(1,userId);
        ps.setBoolean(2,true);
        ResultSet rs = ps.executeQuery();
        if (rs.next()){
            dataresource = getDataResource(rs.getInt("dataresource_id"));
        }
        rs.close();
        ps.close();
        return dataresource;
    }
    // END DATA RESOURCES //

    /**
     * @param projectId
     */
    public void removeAllProjectDataSettings(int projectId) {
        try {
            removeProjectDataResources(projectId);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
	
	private double findEnergyScore(byte[] data) {
	    String score = "";
	    double doubleScore = 0.0;
	    String line = "";
	    BufferedReader in = null;
	    PrintStream p = null;
	    try {
	        in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(data)));
		    while ((line = in.readLine()) != null) {
		        line = line.trim().toLowerCase();
		        if (line.startsWith("best energy score")) {
		            String[] scoreArr = line.split(":");
		            score = scoreArr[1].trim();
		        }
		    }
		    doubleScore = Double.parseDouble(score);
	    } catch (Exception e) {
	        e.printStackTrace();
	    }
	    p.flush();
	    p.close();
	    
	    return doubleScore;
	}
	
	/**
	 * @param applicationId
	 * @return
	 * @throws Exception
	 */
	public ProjectFile getApplication(int applicationId) throws Exception {
		Blob b = null;
		ProjectFile application = null;

		PreparedStatement ps = conn.prepareStatement(selectApplication);
		ps.setInt(1,applicationId);
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			application = new ProjectFile();
			b = rs.getBlob("data");
			application.setData(b.getBytes(1,Integer.parseInt(""+b.length())));
		}
		rs.close();
		ps.close();
		return application;
	}
	
	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public long getDeadline(int projectId) throws Exception {
		long deadline = -1;
		
		PreparedStatement ps = conn.prepareStatement(selectDeadline);
		ps.setInt(1,projectId);
		ps.executeQuery();
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			deadline = rs.getLong("deadline");
		}
		rs.close();
		ps.close();
		
		return deadline;
	}
	
	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public String getBudget(int projectId) throws Exception {
		String budget = null;
		
		PreparedStatement ps = conn.prepareStatement(selectBudget);
		ps.setInt(1,projectId);
		ps.executeQuery();
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			budget = rs.getString("budget");
		}
		rs.close();
		ps.close();
		
		return budget;
	}
	
	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public String getOptimisation(int projectId) throws Exception {
		String optimisation = null;
		
		PreparedStatement ps = conn.prepareStatement(selectOptimisation);
		ps.setInt(1,projectId);
		ps.executeQuery();
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			optimisation = rs.getString("optimisation");
		}
		rs.close();
		ps.close();
		
		return optimisation;
	}

	/**
	 * @param projectId
	 * @return
	 * @throws Exception
	 */
	public boolean QoSexists(int projectId) throws Exception {
		boolean exists = false;
		
		PreparedStatement ps = conn.prepareStatement(checkQoS);
		ps.setInt(1,projectId);
		ps.executeQuery();
		ResultSet rs = ps.executeQuery();
		if (rs.next()){
			exists = true;
		}
		rs.close();
		ps.close();
		
		return exists;
	}
	
	/**
	 * @param projectId
	 * @throws Exception
	 */
	public void removeQoS(int projectId) throws Exception {
		PreparedStatement ps = conn.prepareStatement(deleteQoS);
		ps.setInt(1,projectId);
		ps.execute();
		ps.close();
	}
	
	/**
	 * @param projectId
	 * @param deadline
	 * @param budget
	 * @param optimisation
	 * @throws Exception
	 */
	public void setQoS(int projectId, long deadline, String budget, String optimisation) throws Exception {
		if (deadline >= 0 && budget != null && optimisation != null) {
			PreparedStatement ps = conn.prepareStatement(insertQoS);
			ps.setInt(1,projectId);
			ps.setLong(2,deadline);
			ps.setString(3,budget);
			ps.setString(4,optimisation);
			ps.execute();
			ps.close();
		}
	}
}
