/*
 * Title        :  DBQuerier.java
 * Package      :  org.gridbus.biogrid.data
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  Apr 28, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.data;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import org.globus.ftp.MlsxEntry;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridbus.broker.services.compute.globus.GridFTPUtil;
import org.gridbus.broker.util.BrokerUtil;
import org.gridlab.gridsphere.portlet.User;
import org.ietf.jgss.GSSCredential;


/**
 * @author hag
 *
 */
public class DataQuerier {

    private GSSCredential proxy;
    private String hostname;
    private GridFTPUtil gftp;
    
	/**
	 * @throws Exception
	 */
	public DataQuerier(String hostname, GSSCredential proxy) throws Exception {
        this.hostname = "gsiftp://"+hostname;
        this.proxy = proxy;
        connect();
	}
    
    public DataQuerier(DataResource host, User user) throws Exception {
        this(host.getHostname(),BiogridUtil.getUserProxy(user));
    }
    
    public DataQuerier(String hostname, User user) throws Exception {
        this(hostname,BiogridUtil.getUserProxy(user));
    }
    
	private void connect() throws Exception{
		gftp = new GridFTPUtil(hostname, proxy);
	}

	/**
	 * Closes the database connection
	 * @throws Exception
	 */
	public void disconnect() throws Exception{
		gftp.close();
	}

    public Vector list(String path, String type) throws Exception {
        String oldDir = getCurrentDirectory();//gftp.getCurrentDir();
        gftp.changeDir(path,false);
        System.out.println("--- "+gftp.getCurrentDir());
        Vector list = gftp.mlsd();
        Vector newList = new Vector();
        for (int i=0;i<list.size();i++) {
            MlsxEntry file = (MlsxEntry)list.elementAt(i);
            if (!(""+file).trim().equals("null") && file.get(MlsxEntry.TYPE).equals(type)) {
                newList.add(file);
            }
        }
        gftp.changeDir(oldDir,false);
        return newList;
    }
    
    public Vector listNames(String path, String type) throws Exception {
        Vector list = list(path,type);
        Vector newList = new Vector();
        for (int i=0;i<list.size();i++) {
            MlsxEntry file = (MlsxEntry)list.elementAt(i);
            newList.add(file.getFileName());
        }
        return newList;
    }
    
    public int count(String path, String type) throws Exception {
        return list(path,type).size();
    }

    public void createDir(String path, String newDir) throws Exception {
        String oldDir = getCurrentDirectory();//gftp.getCurrentDir();
        String target = path;
        if (newDir != null) {
            target = BrokerUtil.combinePath(path,newDir);        
        }
        System.out.println("creating directory : "+target);
        gftp.changeDir(target,true);
        System.out.println("current dir (dq) : "+gftp.getCurrentDir());
        gftp.changeDir(oldDir,false);
        System.out.println("current dir (dq) : "+gftp.getCurrentDir());
    }

    public void putFile(String path, String src, String dest) throws Exception {
        putFile(path,src,dest,false);
    }

    public void putFile(String path, InputStream src, String dest) throws Exception {
        putFile(path,src,dest,false);
    }
    
    public void putFile(String path, String src, String dest, boolean createdir) throws Exception {
        if (createdir)
            createDir(path,null);
        gftp.putFile(src,BrokerUtil.combinePath(path,dest));
    }

    public void putFile(String path, InputStream src, String dest, boolean createdir) throws Exception {
        if (createdir)
            createDir(path,null);
        gftp.putFile(src,BrokerUtil.combinePath(path,dest));
    }
    
    public void getFile(String path, String src, String dest) throws Exception {
        gftp.getFile(src,BrokerUtil.combinePath(path,dest));
    }
    
    public String getFile(String path, String src) throws Exception {
        return gftp.getFile(BrokerUtil.combinePath(path,src)).toString();
    }
    
    public OutputStream getFileStream(String path, String src) throws Exception {
        return gftp.getFile(BrokerUtil.combinePath(path,src));
    }
    
    
    public void deleteFile(String path, String file) throws Exception {
        gftp.deleteFile(BrokerUtil.combinePath(path,file));
    }
    
    public void rename(String src, String dest) throws Exception {
        gftp.rename(src, dest);
    }

    public String getCurrentDirectory() throws Exception {
        return gftp.getCurrentDir();
    }
    // FIXME : Add method "public boolean checkUnique(path,name) { }"
    
    public boolean exists(String path,String file) throws Exception {
        String filepath = path;
        if (file != null) {
            filepath = BrokerUtil.combinePath(path,file);
        }
        System.out.println("----- checking path exists : "+filepath);
        return gftp.exists(filepath);
    }
    
    public GridFTPUtil getClient() {
        return this.gftp;
    }
}
