/*
 * Title        :  ProjectataResources.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  May 3, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets;

import java.io.IOException;
import java.util.Collection;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.data.DataQuerier;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridlab.gridsphere.portlet.User;


/**
 * A simple portlet for managing data resources.
 *
 */
public class ProjectDataResources extends GenericPortlet {
	
	/**
	 * Add / Remove resources here.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
		// get the session
    	PortletSession session = request.getPortletSession(true);
		String action = request.getParameter("action");
		String dataresourceId = request.getParameter("dataresourceId");
        String default_dataresourceId = request.getParameter("default_dataresourceId");
		
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;
        DataQuerier dq = null;
        
        User user = BiogridUtil.getUser(request);
        
		if (action.equals("set")) {
			try {
				//add to db
				querier = new DBQuerier();
                int drId = Integer.parseInt(dataresourceId);
                if (drId > 0) {
                    querier.removeAllProjectDataSettings(Integer.parseInt(projectId));
                    querier.setProjectDataResource(Integer.parseInt(projectId),drId);
                }
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (action.equals("default")) {
            try {
                querier = new DBQuerier();
                int drId = Integer.parseInt(default_dataresourceId);
                if (drId > 0) {
                    querier.setDefaultUserDataResource(user.getUserID(),Integer.parseInt(default_dataresourceId));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
		try {
			querier.disconnect();
		} catch (Exception e) {
		}
	}
    
	/**
	 * Get the current list of data resources of the Gridbus Broker.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// get the session
		PortletSession session = request.getPortletSession(true);
		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;		
        User user = BiogridUtil.getUser(request);
        
		if (projectId != null) {
			Collection dataresources = new Vector();
            DataResource dataresource;
            DataResource default_dataresource;
            int dataresourceId = -1;
            int default_dataresourceId = -1;
            
			try {
				querier = new DBQuerier();		
				dataresources = querier.getDataResources(user.getUserID());
                
                dataresource = querier.getProjectDataResource(Integer.parseInt(projectId));
                if (dataresource != null) {
                    dataresourceId = dataresource.getId();
                }
                
                default_dataresource = querier.getDefaultUserDataResource(user.getUserID());
                if (default_dataresource != null) {
                    default_dataresourceId = default_dataresource.getId();
                }
                
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				querier.disconnect();
			} catch (Exception e) {
			}
            
            request.setAttribute("default_dataresourceId", Integer.toString(default_dataresourceId));
            request.setAttribute("dataresourceId", Integer.toString(dataresourceId));
			request.setAttribute("dataresources", dataresources);
			request.setAttribute("url", response.createActionURL().toString());
	
			getPortletContext().getRequestDispatcher("/jsp/projectdataresources.jsp").include(request, response);
		} else {
			getPortletContext().getRequestDispatcher("/jsp/selectprojecterror.jsp").include(request, response);
		}
	}
}
