/*
 * Title        :  ProjectInfo.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  May 3, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.portlets;

import java.io.IOException;
import java.util.Date;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.biogrid.data.DBConnection;
import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.pdb.Project;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.GridbusFarmingEngine;
import org.gridbus.broker.common.Job;
import org.gridbus.broker.constants.ApplicationStatus;
import org.gridbus.broker.constants.Constants;
import org.gridbus.broker.constants.JobStatus;
import org.gridbus.broker.constants.JobType;
import org.gridbus.broker.server.BrokerServer;
import org.gridlab.gridsphere.portlet.User;

/**
 * Very simple portlet for displaying project info. (ie. project name)
 * 
 * @author hag
 */
public class ProjectInfo extends GenericPortlet {

	/**
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
    }
    
    /**
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get the session
    	PortletSession session = request.getPortletSession(true);
		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        
		// quick project stats
		String status = "not initialised";
		String percentJobsCompleted = "N/A";
		String percentBudgetSpent = "N/A";
		Date timeStarted = null;
		
		if (projectId != null) {
	    	DBQuerier querier = null;
	    	Project project = null;
	    	
			try {
				querier = new DBQuerier(new DBConnection());
				// get project
				project = querier.getProject(Integer.parseInt(projectId));
				// get the user
		    	String username = request.getRemoteUser();
                // create/get broker server
                BrokerServer broker = BrokerServer.getInstance();
                // get application
                ApplicationContext app = broker.getApplication(username,projectId);
                
		    	// if fe exists get it out so we can get its details.
		    	if (app != null) {
		    	    // get status
		    	    status = ApplicationStatus.stringValue(app.getStatus());
	        	    // get time started
	        		timeStarted = new Date(app.getStartTime());
                    long successjobs = broker.getJobCount(app.getId(),JobStatus.DONE,Constants.ANY_SERVER);
                    long failedjobs = broker.getJobCount(app.getId(),JobStatus.FAILED,Constants.ANY_SERVER);
                    long totaljobs = broker.getJobCount(app.getId(),JobStatus.STATUS_ANY,Constants.ANY_SERVER);
                    
		    	    // get percent jobs done
	        	    long donejobs = successjobs + failedjobs;
                    if (totaljobs != 0) {
                        percentJobsCompleted = (100 * donejobs / totaljobs)+" %";
                    }
                    
	        	    // get percent budget spent
//	        		percentBudgetSpent = (100 * fe.getBudgetSpent() / fe.getBudget())+" %";
		    	}

				querier.disconnect();

			} catch (Exception e) {
				e.printStackTrace();
			}

	        request.setAttribute("url", response.createActionURL().toString());
	        request.setAttribute("project", project);
	        request.setAttribute("status", status);
	        request.setAttribute("percentJobsCompleted", percentJobsCompleted);
	        request.setAttribute("percentBudgetSpent", percentBudgetSpent);
            
	        if (timeStarted != null) {
	            request.setAttribute("timeStarted", timeStarted.toString());
	        } else {
		        request.setAttribute("timeStarted", "N/A");	            
	        }
	        
            if (project != null) {
                getPortletContext().getRequestDispatcher("/jsp/projectinfo.jsp").include(request, response);
            } else {
                getPortletContext().getRequestDispatcher("/jsp/selectprojecterror.jsp").include(request, response);
            }
		} else {
	        getPortletContext().getRequestDispatcher("/jsp/selectprojecterror.jsp").include(request, response);
		}
    }

}
