/*
 * Title        :  ProjectResources.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  May 3, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets;

import java.io.IOException;
import java.util.Collection;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridlab.gridsphere.portlet.User;


/**
 * A simple portlet for managing resources.  Allows user to assign available resources
 * to the current project for use during execution.
 *
 */
public class ProjectResources extends GenericPortlet {
	
	/**
	 * Add / Remove resources here.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
		// get the session
    	PortletSession session = request.getPortletSession(true);
		String action = request.getParameter("action");
		String[] resourceIds = request.getParameterValues("resourceId");
		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;
		
		if (action.equals("set")) {
			try {
				//add to db
				querier = new DBQuerier();
				querier.removeProjectResources(Integer.parseInt(projectId));
				if (resourceIds != null) {
                    for (int i=0; i<resourceIds.length; i++) {
    					querier.setProjectResource(Integer.parseInt(projectId),Integer.parseInt(resourceIds[i]));
    				}
                }
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		try {
			querier.disconnect();
		} catch (Exception e) {
		}
	}
	
	/**
	 * Get the current list of resources of the Gridbus Broker.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// get the session
		PortletSession session = request.getPortletSession(true);
		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;		
        User user = BiogridUtil.getUser(request);
        
		if (projectId != null) {
			Collection projectResources = new Vector();
			Collection resources = new Vector();
			
			try {
				querier = new DBQuerier();		
				resources = querier.getResources(user.getUserID());
				projectResources = querier.getProjectResources(Integer.parseInt(projectId));
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				querier.disconnect();
			} catch (Exception e) {
			}
			request.setAttribute("projectResources", projectResources);
			request.setAttribute("resources", resources);
			request.setAttribute("url", response.createActionURL().toString());
	
			getPortletContext().getRequestDispatcher("/jsp/projectresources.jsp").include(request, response);
		} else {
			getPortletContext().getRequestDispatcher("/jsp/selectprojecterror.jsp").include(request, response);
		}
	}
}
