/*
 * Title        :  ProjectResources.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  May 3, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets;

import java.io.IOException;
import java.util.Collection;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridlab.gridsphere.portlet.User;


/**
 * A simple portlet for managing resources.  Allows user to assign available resources
 * to the current project for use during execution.
 *
 */
public class ProjectResourcesAdmin extends GenericPortlet {
	
	/**
	 * Add / Remove resources here.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
		// get the session
    	PortletSession session = request.getPortletSession(true);
		String action = request.getParameter("action");
        String hostname = request.getParameter("hostname");
        String contact = request.getParameter("contact");
        String type = request.getParameter("type");
        String price = request.getParameter("price");
        String queue = request.getParameter("queue");
        String joblimit = request.getParameter("joblimit");
        String dataresourceId = request.getParameter("dataresourceId");
        String resourceId = request.getParameter("resourceId");

		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;
		
        User user = BiogridUtil.getUser(request);
        
		if (action.equals("add")) {
			try {
				//add to db
				querier = new DBQuerier();
                querier.addResource(user.getUserID(), hostname, contact, queue, Integer.parseInt(type), Float.parseFloat(price), Integer.parseInt(dataresourceId), Integer.parseInt(joblimit));
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else if (action.equals("remove")) {
            try {
                //remove from db (and remove all links to it).
                querier = new DBQuerier();
                querier.removeResource(user.getUserID(), Integer.parseInt(resourceId));
                querier.removeResourceProjects(Integer.parseInt(resourceId));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
		try {
			querier.disconnect();
		} catch (Exception e) {
		}
	}
	
	/**
	 * Get the current list of resources of the Gridbus Broker.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
	public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
		// get the session
		PortletSession session = request.getPortletSession(true);
		String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
		DBQuerier querier = null;		
        User user = BiogridUtil.getUser(request);
        
		Collection resources = new Vector();
        Collection dataresources = new Vector();

        try {
			querier = new DBQuerier();		
			resources = querier.getResources(user.getUserID());
            dataresources = querier.getDataResources(user.getUserID());
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			querier.disconnect();
		} catch (Exception e) {
		}
		request.setAttribute("resources", resources);
        request.setAttribute("dataresources", dataresources);
		request.setAttribute("url", response.createActionURL().toString());

		getPortletContext().getRequestDispatcher("/jsp/projectresourcesadmin.jsp").include(request, response);
	}
}
