/*
 * Title        :  SimpleProjectRunAPI.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  GridbusPortlets
 * Description	:  		
 * Created on   :  March, 2007
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.portlets;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.data.DataQuerier;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.biogrid.pdb.Resource;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.ComputeServer;
import org.gridbus.broker.common.JobQueue;
import org.gridbus.broker.common.Qos;
import org.gridbus.broker.common.WorkUnit;
import org.gridbus.broker.common.security.LocalProxyCredential;
import org.gridbus.broker.constants.ScheduleOptimisationType;
import org.gridbus.broker.server.BrokerServer;
import org.gridbus.broker.services.compute.ServerFactory;
import org.gridbus.broker.services.compute.globus.GlobusComputeServer;
import org.gridbus.broker.util.BrokerUtil;
import org.ietf.jgss.GSSCredential;

/**
 * Executes the experiment using the API to create Jobs.
 * 
 * Uses the Broker API to create Job objects based on user input and passes these objects 
 * to the Broker for execution.
 * 
 */
public class SimpleProjectRunAPI extends GenericPortlet {

	/**
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
    	String action = request.getParameter("action");
        // collect error/info msgs here to report to user
        Collection msgs = new ArrayList();
        if (action != null) {
    	    try {
                // get/create broker
    	        if (action.equals("run")) {
                    msgs = runProject(request,response); // pass parameters
                    if (msgs==null || msgs.size() == 0) {
                        request.setAttribute("success", "Success. Application Running...");
                    }
                } else if (action.equals("check")) {
                    // check if output files are available for download
                    checkDownloads(request, response);
                }
    	    } catch (Exception e) {
                e.printStackTrace(); //kna added : Nov 17, 2005
        	}
            if (msgs.size() > 0)
                response.setRenderParameter("msgs",(String[])msgs.toArray(new String[0]));
        }
    }
    
    /**
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        
        String user = request.getRemoteUser();
        DBQuerier querier = null;
        
        String page = "/jsp/simpleprojectrun.jsp";

        Collection resources = new Vector();
        Collection dataresources = new Vector();
        String[] output = request.getParameterValues("output");
        if (output == null) output = new String[0];
        boolean[] download = new boolean[output.length]; //size of outputs
        for (int i=0;i<download.length;i++) { download[i]=false; }
        
        try {
            querier = new DBQuerier();      
            resources = querier.getResources(user);
            dataresources = querier.getDataResources(user);
            
            // check if output files are available so they can be downloaded
            String s_dataresourceId = request.getParameter("dataresourceId");
            if (s_dataresourceId!=null && !"".equals(s_dataresourceId)) {
                int dataresourceId = Integer.parseInt(s_dataresourceId);
                if (dataresourceId > 0) {
                    DataResource dr = querier.getDataResource(dataresourceId);
                    DataQuerier dq = new DataQuerier(dr,BiogridUtil.getUser(user));
                    String outputPath = BrokerUtil.combinePath(dr.getHomedir(),request.getParameter("outputdir"));
                    for (int i=0; i<output.length; i++) {
                        if (!"".equals(output[i]) && dq.exists(outputPath,output[i])) {
                            download[i] = true;
                        } else {
                            download[i] = false;
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            querier.disconnect();
        } catch (Exception e) {
        }

        // handle request params
        request.setAttribute("resources", resources);
        request.setAttribute("dataresources", dataresources);
        request.setAttribute("resourceId", request.getParameter("resourceId"));
        request.setAttribute("dataresourceId", request.getParameter("dataresourceId"));
        request.setAttribute("command", request.getParameter("command"));
        String[] input = request.getParameterValues("input");
        if (input == null) input = new String[0];
        request.setAttribute("input", input);
        request.setAttribute("download", download);
        request.setAttribute("output", output);
        request.setAttribute("outputdir", request.getParameter("outputdir"));

        request.setAttribute("user", user);
        
        String[] msgs = request.getParameterValues("msgs");
        if (msgs == null) msgs = new String[0];
        request.setAttribute("msgs",msgs);
        request.setAttribute("url", response.createActionURL().toString());
        getPortletContext().getRequestDispatcher(page).include(request, response);
    }
    
    /**
     * 
     * Just repost parameters from the request
     * 
     * @param request
     * @param response
     */
    private void checkDownloads(ActionRequest request, ActionResponse response) {
        // get all input parameters, server, cmd, inputs, outputs etc.
        String s_resourceId = request.getParameter("resourceId");
        int resourceId = Integer.parseInt(s_resourceId);
        String command = request.getParameter("command");
        String[] input = request.getParameterValues("input");
        String s_dataresourceId = request.getParameter("dataresourceId");
        int dataresourceId = Integer.parseInt(s_dataresourceId);
        String outputdir = request.getParameter("outputdir");
        String[] output = request.getParameterValues("output");
        
        response.setRenderParameter("resourceId", ""+resourceId);
        response.setRenderParameter("command", command);
        response.setRenderParameter("input", input);
        response.setRenderParameter("dataresourceId", ""+dataresourceId);
        response.setRenderParameter("outputdir", outputdir);
        response.setRenderParameter("output", output);
    }
    
    private Collection runProject(ActionRequest request, ActionResponse response) throws Exception {

        Collection errMsgs = new ArrayList();

        try {
            PortletSession session = request.getPortletSession(true);
            String s_projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
            int projectId = Integer.parseInt(s_projectId);
            String username = request.getRemoteUser();
            
            // get all input parameters, server, cmd, inputs, outputs etc.
            String s_resourceId = request.getParameter("resourceId");
            int resourceId = Integer.parseInt(s_resourceId);
            String command = request.getParameter("command");
            String[] input = request.getParameterValues("input");
            String s_dataresourceId = request.getParameter("dataresourceId");
            int dataresourceId = Integer.parseInt(s_dataresourceId);
            String outputdir = request.getParameter("outputdir");
            String[] output = request.getParameterValues("output");
            
            response.setRenderParameter("resourceId", ""+resourceId);
            response.setRenderParameter("command", command);
            response.setRenderParameter("input", input);
            response.setRenderParameter("dataresourceId", ""+dataresourceId);
            response.setRenderParameter("outputdir", outputdir);
            response.setRenderParameter("output", output);
            
            DBQuerier querier = new DBQuerier();
    
            // Validate QoS Parameters
            long deadline = querier.getDeadline(projectId);
            String budget = querier.getBudget(projectId);
            String optimisation = querier.getOptimisation(projectId);
            if (deadline == -1 || budget == null || optimisation == null)
                errMsgs.add("QoS requirements must be set before running experiment.");
            if (deadline < System.currentTimeMillis())
                errMsgs.add("Invalid QoS, Deadline already past.");
            
            // Validate resource
            Collection resources = null;
            if (resourceId < 0) {
                resources = querier.getProjectResources(projectId);
            } else {
                resources = new ArrayList();
                resources.add(querier.getResource(resourceId));
            }
            
            // Validate command
            if (command==null || "".equals(command)) {
                errMsgs.add("Command field cannot be empty.");
            }
            
            // Validate data resource and output dir
            if (dataresourceId < 0) {
                errMsgs.add("No valid data server selected for outputs");
            }
            
            // Create directory for Results
            DataResource dr = querier.getDataResource(dataresourceId);
            DataQuerier dq = new DataQuerier(dr,BiogridUtil.getUser(username));
            String outputPath = BrokerUtil.combinePath(dr.getHomedir(),outputdir);
            dq.createDir(outputPath,null);
            
            // Get path to GFTP for staging files
            String outputURLWithPath = "gsiftp://"+BrokerUtil.combinePath(dr.getHostname()+":2811",outputPath);
            if (!dq.exists(outputPath,null)) {
                errMsgs.add("Could not create output directory "+outputURLWithPath);
            }
            
            // Check proxy credential before we run
            GSSCredential credential = BiogridUtil.getUserProxy(BiogridUtil.getUser(username));
            if (credential.getRemainingLifetime() <= 0) {
                errMsgs.add("Credentials expired. Renew Credentials and try again.");
            }
            
            if (projectId > 0 // we have a project
                    && credential != null // we have a proxy
                    && errMsgs.size() <= 0) {  // experiment has been validated successfully
                
                // get broker
                BrokerServer broker = BrokerServer.getInstance();
    
                // get app, if it already exists
                ApplicationContext app = broker.getApplication(username,""+projectId);
                if (app != null) {
                    // STOP PREVIOUS APPLICATION
                    broker.stopApplication(app.getId());
                }
    
                app = new ApplicationContext(username);
                app.setUsername(username);
                app.setName(""+projectId);
                broker.updateApplication(app);
    
                WorkUnit job = BiogridUtil.createWorkUnit(projectId, command, input, output, outputURLWithPath);
                broker.addJob(app.getId(), job);
                
                Qos qos = app.getQos();
                qos.setBudget(Long.parseLong(budget));
                qos.setDeadline(deadline);
                qos.setOptimisation(ScheduleOptimisationType.NONE);
                app.setQos(qos);
                
                broker.updateQos(app.getId(),qos);
    
                // create credentials
                LocalProxyCredential pc = new LocalProxyCredential();
                pc.setId("pc1");
                pc.setProxy((GSSCredential) credential);
                broker.addCredential(app.getId(),pc);
               
                // add resources to farming engine
                for (Iterator it=resources.iterator();it.hasNext();) {
                    
                    Resource resource = (Resource)it.next();
                    
                    ComputeServer cs = ServerFactory.getComputeServer(resource.getType(),resource.getHostname());
                    String jm = BiogridUtil.getJobManager(resource.getContact());
                    
                    cs.setName(resource.getHostname());
                    if (cs instanceof GlobusComputeServer) {
                        GlobusComputeServer gcs = (GlobusComputeServer)cs;
                        gcs.setJobManager(jm);
                        DataResource dataresource = resource.getDataResource();
                        String stagingServer = null;
                        if (dataresource != null) {
                            stagingServer = dataresource.getHostname()+"/"+dataresource.getHomedir()+"/";
                        } else {
                            stagingServer = resource.getHostname()+"/~/";
                        }
                        gcs.setFileStagingURL("gsiftp://"+stagingServer);
                    }
                    
                    // need to add support for queues
                    cs.setPricePerCpuSec(resource.getPrice());
                    cs.setPricePerJob(resource.getPrice());
                    cs.setJobLimit(resource.getJoblimit());
    
                    if (resource.getQueue() != null && !"".equals(resource.getQueue())) {
                        JobQueue queue = new JobQueue(resource.getQueue());
                        queue.setLimit(resource.getJoblimit());
                        List queues = cs.getQueues();
                        if (queues == null) {
                            queues = new ArrayList();
                        }
                        queues.add(queue);
                        cs.setQueues(queues);
                    }
    
                    broker.addService(app.getId(),cs);
                }
    
                broker.startApplication(app.getId());
            } else {
                errMsgs.add("Experiement could not be started. Please correct errors and try again...");
            }
        } catch (Exception e) {
            errMsgs.add("Unhandled Error: "+e.getMessage());
            e.printStackTrace();
        }
        return errMsgs;
    }
    
}


