/*
 * Title        :  SimpleProjectRunXPML.java
 * Package      :  org.gridbus.biogrid.portlets
 * Project      :  GridbusPortlets
 * Description	:  		
 * Created on   :  March, 2007
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.portlets;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.portlet.PortletFileUpload;
import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.biogrid.pdb.Resource;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.ComputeServer;
import org.gridbus.broker.common.JobQueue;
import org.gridbus.broker.common.Qos;
import org.gridbus.broker.common.security.LocalProxyCredential;
import org.gridbus.broker.constants.ScheduleOptimisationType;
import org.gridbus.broker.parsers.xpml.XPMLParser;
import org.gridbus.broker.server.BrokerServer;
import org.gridbus.broker.services.compute.ServerFactory;
import org.gridbus.broker.services.compute.globus.GlobusComputeServer;
import org.ietf.jgss.GSSCredential;

/**
 * Executes the experiment using XPML (XML) application description input file.
 * 
 * Allows user to upload and edit an XML representation of an application to run on the Grid.
 * 
 * The Broker API is then used to parse the XPML and generate Grid jobs which are then run on the specified resources.
 */
public class SimpleProjectRunXPML extends GenericPortlet {

	/**
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
    	
        // get the session
        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String username = request.getRemoteUser();
        
        String action = request.getParameter("action");
        String xpml = request.getParameter("xpml");
        
        // collect error/info msgs here to report to user
        Collection msgs = new ArrayList();

        System.out.println("--- Processing XPML ");

        try {
            if (action != null) {
                    // get/create broker
        	        if (action.equals("run")) {
                        msgs = runProject(Integer.parseInt(projectId),username); // pass parameters
                        if (msgs==null || msgs.size() == 0) {
                            request.setAttribute("success", "Success. Application Running...");
                        }
                    } else if (action.equals("update")) {
                        // save xpml to file.
                        File file = getXPMLFile(username,Integer.parseInt(projectId));
                        updateXPML(file,xpml);
                    }
                    
            } else {
                // upload XPML file
                DiskFileItemFactory df=new DiskFileItemFactory();
                df.setSizeThreshold(1000);
                PortletFileUpload myupload = new PortletFileUpload(df); 
                List fileItems=myupload.parseRequest(request,request.getContentType(),request.getContentLength(),request.getPortletInputStream());
                for(int i=0;i<fileItems.size();i++){
                    FileItem tmpfile=(FileItem)fileItems.get(i);
                    if(tmpfile.getFieldName().equals("action")) {
                        action = ((FileItem)fileItems.get(i)).getString();
                    }
                    if (tmpfile.getName() != null && !tmpfile.getName().equals("") && tmpfile.getContentType() != null) {
                        updateXPML(getXPMLFile(username,Integer.parseInt(projectId)), new String(tmpfile.get()));
                    }
                }
            }
        
	    } catch (Exception e) {
            e.printStackTrace(); //kna added : Nov 17, 2005
    	}
        
        if (msgs.size() > 0)
            response.setRenderParameter("msgs",(String[])msgs.toArray(new String[0]));
    }
    
    /**
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String xpml = "";
        
        String user = request.getRemoteUser();
        
        String page = "/jsp/simpleprojectrunxpml.jsp";

        // get contents of xpml file
        try {
            File file = getXPMLFile(user, Integer.parseInt(projectId));
            xpml = BiogridUtil.readFile(file);
        } catch (Exception e){
            page = "/jsp/selectprojecterror.jsp";
        }
        String[] msgs = request.getParameterValues("msgs");
        request.setAttribute("msgs",msgs);
        request.setAttribute("xpml",xpml);
        request.setAttribute("url", response.createActionURL().toString());
        getPortletContext().getRequestDispatcher(page).include(request, response);
    }
    
    private File getXPMLFile(String username, int projectId) throws IOException {
        File xpml = new File(username+"_"+projectId+".xml");
        if (!xpml.exists()) {
            xpml.createNewFile();
        }
        System.out.println("Path to xpml is : "+xpml.getAbsolutePath());
        return xpml;
    }
    
    private void updateXPML(File file, String xpml) throws Exception {
        file.delete();
        file.createNewFile();
        
        FileOutputStream out = new FileOutputStream(file);
        out.write(xpml.getBytes());
        
        System.out.println("updating xpml:\n"+xpml);
        out.flush();
        out.close();
    }
    
    private Collection runProject(int projectId, String username) {

        Collection errMsgs = new ArrayList();
        
        try {
            DBQuerier querier = new DBQuerier();
            
            // Validate QoS Parameters
            long deadline = querier.getDeadline(projectId);
            String budget = querier.getBudget(projectId);
            String optimisation = querier.getOptimisation(projectId);
            if (deadline == -1 || budget == null || optimisation == null)
                errMsgs.add("QoS requirements must be set before running experiment.");
            if (deadline < System.currentTimeMillis())
                errMsgs.add("Invalid QoS, Deadline already past.");
            
            // Validate Resources
            Collection resources = querier.getProjectResources(projectId);
            if (resources == null) {
                errMsgs.add("Resources for execution have not been configured.");
            }
            
            File xpml = getXPMLFile(username, projectId);
            
            // Check proxy credential before we run
            GSSCredential credential = BiogridUtil.getUserProxy(BiogridUtil.getUser(username));
            if (credential.getRemainingLifetime() <= 0) {
                errMsgs.add("Credentials expired. Renew Credentials and try again.");
            }
            
            if (projectId > 0 // we have a project
                    && credential != null // we have a proxy
                    && errMsgs.size() <= 0  // experiment has been validated successfully
                    && xpml.exists()) { // xpml has been created successfully
                            
                // get broker
                BrokerServer broker = BrokerServer.getInstance();
    
                // get app, if it already exists
                ApplicationContext app = broker.getApplication(username,""+projectId);
                if (app != null) {
                    // STOP PREVIOUS APPLICATION
                    broker.stopApplication(app.getId());
                }
                
                //now create the new app
                XPMLParser parser = new XPMLParser();
                parser.parseApplication(xpml.getAbsolutePath(), broker);
                parser.parseWorkUnits(xpml.getAbsolutePath(), broker);
                
                app = broker.getApplication(parser.getApplicationId());
                app.setUsername(username);
                app.setName(""+projectId);
                broker.updateApplication(app);
                
                Qos qos = app.getQos();
                qos.setBudget(Long.parseLong(budget));
                qos.setDeadline(deadline);
                qos.setOptimisation(ScheduleOptimisationType.NONE);
                app.setQos(qos);
                
                broker.updateQos(app.getId(),qos);
    
                // create credentials
                LocalProxyCredential pc = new LocalProxyCredential();
                pc.setId("pc1");
                pc.setProxy((GSSCredential) credential);
                broker.addCredential(app.getId(),pc);
               
                // add resources to farming engine
                for (Iterator it=resources.iterator();it.hasNext();) {
                    
                    Resource resource = (Resource)it.next();
                    
                    ComputeServer cs = ServerFactory.getComputeServer(resource.getType(),resource.getHostname());
                    String jm = BiogridUtil.getJobManager(resource.getContact());
                    
                    cs.setName(resource.getHostname());
                    if (cs instanceof GlobusComputeServer) {
                        GlobusComputeServer gcs = (GlobusComputeServer)cs;
                        gcs.setJobManager(jm);
                        DataResource dataresource = resource.getDataResource();
                        String stagingServer = null;
                        if (dataresource != null) {
                            stagingServer = dataresource.getHostname()+"/"+dataresource.getHomedir()+"/";
                        } else {
                            stagingServer = resource.getHostname()+"/~/";
                        }
                        gcs.setFileStagingURL("gsiftp://"+stagingServer);
                    }
                    
                    // need to add support for queues
                    cs.setPricePerCpuSec(resource.getPrice());
                    cs.setPricePerJob(resource.getPrice());
                    cs.setJobLimit(resource.getJoblimit());
    
                    if (resource.getQueue() != null && !"".equals(resource.getQueue())) {
                        JobQueue queue = new JobQueue(resource.getQueue());
                        queue.setLimit(resource.getJoblimit());
                        List queues = cs.getQueues();
                        if (queues == null) {
                            queues = new ArrayList();
                        }
                        queues.add(queue);
                        cs.setQueues(queues);
                    }
    
                    broker.addService(app.getId(),cs);
                }
    
                broker.startApplication(app.getId());
            } else {
                errMsgs.add("Experiement could not be started. Please correct errors and try again...");
            }
        } catch (Exception e) {
            errMsgs.add("Unhandled Error: "+e.getMessage());
            e.printStackTrace();
        }

        return errMsgs;
    }
    
}


