/*
 * Title            :  GBBMonitor.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets.gridbus;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.ComputeServer;
import org.gridbus.broker.common.GridbusFarmingEngine;
import org.gridbus.broker.common.Job;
import org.gridbus.broker.common.ParameterSweepApplicationContext;
import org.gridbus.broker.constants.ApplicationStatus;
import org.gridbus.broker.constants.Constants;
import org.gridbus.broker.constants.JobStatus;
import org.gridbus.broker.constants.ServiceType;
import org.gridbus.broker.exceptions.GridBrokerException;
import org.gridbus.broker.server.BrokerServer;
import org.gridbus.broker.util.BrokerUtil;


/**
 * A simple portlet for displaying the execution status of the Gridbus Broker.  This
 * really just passes the set of jobs to the view where each of the job's statuses are
 * printed.
 */
public class GBBMonitor extends GenericPortlet {
	public static final String APPLICATION_ID = "applicationID";
	/**
	 * @param req
	 * @param res
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {

        // get the session
    	PortletSession session = req.getPortletSession(true);
        String action = req.getParameter("action");
        String pageId = req.getParameter("pageId");
        
        if (action!=null && action.equals("view")) {
            if (req.getParameter("jid") != null && !req.getParameter("jid").equals("")) {
                session.setAttribute("jid",req.getParameter("jid"),PortletSession.APPLICATION_SCOPE);
            }
            if (req.getParameter("resourceId") != null && !req.getParameter("resourceId").equals("")) {
                session.setAttribute("resourceId",req.getParameter("resourceId"),PortletSession.APPLICATION_SCOPE);
            }
        } else if (action!=null && action.equals("filter")) {
            session.setAttribute("jstatus",req.getParameter("jstatus"),PortletSession.PORTLET_SCOPE);
        }
        
        // do paging
    	try {
    		Integer.parseInt(pageId);
    	} catch (Exception e) {
    		pageId = null;
    	}
        session.setAttribute("pageId",req.getParameter("pageId"),PortletSession.PORTLET_SCOPE);
        
        // do refresh rate
        if (req.getParameter("txtRefreshtime")!=null && !req.getParameter("txtRefreshtime").equals("")){
            session.setAttribute("refreshtime",req.getParameter("txtRefreshtime"));
        }
    }

    /**
     * Get the current status of the Gridbus Broker.  This includes determining whether or not
     * the broker is actually running as well as printing out the statuses of each job
     * (ie. waiting, pending, submitted, failed, done. etc.)
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

        // get the session
        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String username = request.getRemoteUser();

        BrokerServer broker = null;
        String status = "stopped";
        Collection jobs = new Vector();
        Collection servers = new Vector();
        Collection unreadyServers = null;
        String refreshtime = "10"; 
        int currentPage = 1;
		int maxPage = 1;
        
        //set defaults, in case nothing is selected
        refreshtime = (String) session.getAttribute("refreshtime");
        if (refreshtime==null || refreshtime.trim().equals("")) 
        	refreshtime = "10";
        
        String pageId = (String) session.getAttribute("pageId", PortletSession.PORTLET_SCOPE);
        if (pageId != null) {
            currentPage = Integer.parseInt(pageId);
        }

        try {
	       	// create/get broker server
	       	broker = BrokerServer.getInstance();
            ApplicationContext app = broker.getApplication(username,projectId);
            
	        if (app != null) {
                //set the application id to further request
                session.setAttribute(APPLICATION_ID,app.getId(),PortletSession.APPLICATION_SCOPE);

                // get the current set of jobs
				jobs = broker.getJobs(app.getId());
                Comparator c = new JobComparator();
                Collections.sort((List)jobs,c);
                
				Collection allServers = broker.getServices(app.getId(),ServiceType.COMPUTE);

                if (app instanceof ParameterSweepApplicationContext && allServers != null) {
                    unreadyServers = new Vector();
                    // loop through all servers and if exist in the preprocessed servers leave them in the list
                    // otherwise they should be added to the preServers list
                    ParameterSweepApplicationContext papp = (ParameterSweepApplicationContext)app;
                    // strings of hostnames
                    Collection doneServers = papp.getPreProcessedServers();
                    for (Iterator it=allServers.iterator();it.hasNext();) {
                        ComputeServer cs = (ComputeServer)it.next();
                        if (doneServers.contains(cs.getHostname())) {
                            servers.add(cs);
                        } else {
                            unreadyServers.add(cs);
                        }
                    }
                } else {
                    servers = allServers;
                }
                
				//check the status of the application
				status = ApplicationStatus.stringValue(app.getStatus());
				
				initRequestFields(request);
	        	try{
	        	    Date startTime = new Date(app.getStartTime()); 
	           		long stageInJobs = broker.getJobCount(app.getId(),JobStatus.STAGE_IN,Constants.ANY_SERVER);
	           		long stageOutJobs = broker.getJobCount(app.getId(),JobStatus.STAGE_OUT,Constants.ANY_SERVER);
	           		long scheduledJobs = broker.getJobCount(app.getId(),JobStatus.SCHEDULED,Constants.ANY_SERVER);
	        	    long runningJobs = broker.getJobCount(app.getId(),JobStatus.SUBMITTED,Constants.ANY_SERVER) 
                                            + broker.getJobCount(app.getId(),JobStatus.PENDING,Constants.ANY_SERVER)
                                            + broker.getJobCount(app.getId(),JobStatus.ACTIVE,Constants.ANY_SERVER);
	                long finishedJobs = broker.getJobCount(app.getId(),JobStatus.DONE,Constants.ANY_SERVER);
	                long failedJobs = broker.getJobCount(app.getId(),JobStatus.FAILED,Constants.ANY_SERVER);
	                long unknownJobs = broker.getJobCount(app.getId(),JobStatus.UNKNOWN,Constants.ANY_SERVER);
	           		long totalJobs = broker.getJobCount(app.getId(),JobStatus.STATUS_ANY,Constants.ANY_SERVER);
	        	    long donejobs = finishedJobs + failedJobs;
	        	    long readyJobs = broker.getJobCount(app.getId(),JobStatus.READY,Constants.ANY_SERVER);
	        		float temp = 0;
	        		float totbudget = app.getQos().getBudget();
	        		float budspent = app.getQos().getBudgetSpent();
	        		
	        		if(totalJobs != 0)
	        			temp = 100 * donejobs / totalJobs;
	        		else
	        			temp = 0;
	        		request.setAttribute("jDone", Float.toString(temp));
	        		request.setAttribute("doneJobs", ""+donejobs);
	        		
	        		if(totalJobs != 0)
	        			temp = 100 * (totalJobs - donejobs) / totalJobs;
	        		else
	        			temp = 0;	        		
	      
	        		request.setAttribute("jRemain", Float.toString(temp));
	        		
					String timerem = "0";
					String timetaken = "0";
                    
					if (app.getStatus() != ApplicationStatus.STOPPED){
				        timetaken = BrokerUtil.getHHMMSS(System.currentTimeMillis() - app.getStartTime());
					    //only calculcate time-remaining if the broker is running
					    timerem = BrokerUtil.getHHMMSS(app.getQos().getDeadline() - System.currentTimeMillis());
					}
					if (timetaken.equals("0")) {
					    timetaken = (String) session.getAttribute("timeTaken");
					}
					request.setAttribute("timeTaken", timetaken);
					session.setAttribute("timeTaken", timetaken);
	        		request.setAttribute("timeRemaining", timerem);
	        		request.setAttribute("startTime", startTime);
	        		
	        		if(totbudget !=0)
	        			temp = 100 * budspent / totbudget;
	        		else
	        			temp = 0;
	        		
	        		request.setAttribute("bDone", Float.toString(temp));
	        		
	        		if(totbudget!=0)
	        			temp = 100 * (totbudget - budspent) / totbudget;
	        		else 
	        			temp = 0;
	        		
	        		request.setAttribute("bRemain", Float.toString(temp));
	        		request.setAttribute("bSpent", Float.toString(budspent));
	        		
	        		request.setAttribute("showProgress", "true");
	        		request.setAttribute("totBudget", Float.toString(totbudget));
	        		
	        		request.setAttribute("totalJobs", Long.toString(totalJobs));
	        		request.setAttribute("stageInJobs",Long.toString(stageInJobs));
	        		request.setAttribute("stageOutJobs",Long.toString(stageOutJobs));	        		
	        		request.setAttribute("scheduledJobs",Long.toString(scheduledJobs));
	                request.setAttribute("readyJobs", Long.toString(readyJobs));
	                request.setAttribute("runningJobs", Long.toString(runningJobs));
	                request.setAttribute("finishedJobs", Long.toString(finishedJobs));
	                request.setAttribute("failedJobs", Long.toString(failedJobs));
	                request.setAttribute("unknownJobs", Long.toString(unknownJobs));
	        	}catch(Exception e){
	        		e.printStackTrace();
	        	}
	        }
        } catch (Exception e) {
        	e.printStackTrace();
        }
        request.setAttribute("pageId",currentPage+"");
        request.setAttribute("maxPage",maxPage+"");
        request.setAttribute("status", status);
        request.setAttribute("jobs", jobs);
        request.setAttribute("servers", servers);
        request.setAttribute("unreadyServers", unreadyServers);
        request.setAttribute("url", response.createActionURL().toString());
        request.setAttribute("refreshtime",refreshtime);

        getPortletContext().getRequestDispatcher("/jsp/monitor.jsp").include(request, response);
    }
    
    private void initRequestFields(RenderRequest request){
    	request.setAttribute("showProgress", "false");
    	request.setAttribute("jDone", "0");
    	request.setAttribute("jRemain", "0");
    	request.setAttribute("bDone", "0");
    	request.setAttribute("bRemain", "0");
    	request.setAttribute("totBudget", "0");
    	
    	request.setAttribute("timeRemaining", "0");
    	request.setAttribute("startTime", null);
    	
    	request.setAttribute("totalJobs", "0");
		request.setAttribute("stageInJobs","0");
		request.setAttribute("stageOutJobs","0");
		request.setAttribute("scheduledJobs","0"); 	
        request.setAttribute("readyJobs", "0");
        request.setAttribute("runningJobs", "0");
        request.setAttribute("finishedJobs", "0");
        request.setAttribute("failedJobs", "0");
        request.setAttribute("unknownJobs", "0");    	
    }
    
    //JobComparator for sorting purpose by job's name
    class JobComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            try {
                Job j1 = (Job)o1;
                Job j2 = (Job)o2;

                int id1 = Integer.parseInt(j1.getName().substring(1));
                int id2 = Integer.parseInt(j2.getName().substring(1));

                if (id1 < id2) {
                    return -1;
                } else if (id1 > id2) {
                    return 1;
                } else {
                    return 0;
                }
            } catch (Exception e) {
            }
            return 0;
        }
    }
}
