/*
 * Title            :  GBBDriver.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets.gridbus;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.GregorianCalendar;
import java.util.Locale;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.log4j.Logger;
import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.util.BiogridUtil;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.Broker;
import org.gridbus.broker.common.Qos;
import org.gridbus.broker.constants.ScheduleOptimisationType;
import org.gridbus.broker.scheduler.DBScheduler;
import org.gridbus.broker.scheduler.SchedulerFactory;
import org.gridbus.broker.server.BrokerServer;
import org.gridlab.gridsphere.portlet.User;

/**
 * This portlet allows you to select required scheduling algorithm and Qos settings for the application,
 * run by the Gridbus Broker.
 *
 */
public class GBBQos extends GenericPortlet {

    /**
     * Logger for this class
     */
//    private final Logger logger = Logger.getLogger(GBBQos.class);
    
    /**
     * Process action specific to BioGrid portal.  Writes QoS data to database.
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {

        response.setRenderParameter("qosMsg","");
        
        String setAlgo = request.getParameter("setAlgo");
        String setBudget = request.getParameter("setBudget");
        String setDeadline = request.getParameter("setDeadline");
        String setAll = request.getParameter("setAll");

        if (setAlgo == null && setBudget == null && setDeadline == null && setAll == null)
            return;

        
        DBQuerier querier = null;
        
        // get the session
        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String username = request.getRemoteUser();

        BrokerServer broker = null;
        
        String qosMsg = "";

        String hh = request.getParameter("selHour");
        String mm = request.getParameter("selMin");
        String ss = request.getParameter("selSec");
        String dd = request.getParameter("selDay");
        String mmm = request.getParameter("selMonth");
        String yyyy = request.getParameter("selYear");
        String deadline = dd+" "+mmm+" "+yyyy+" "+hh+":"+mm+":"+ss;

        String budget = request.getParameter("txtBudget");
        String schedAlgo = request.getParameter("selAlgo");
        // get the Gridbus Broker farming engine from the session.
        
        try {
            // create/get broker server
            broker = BrokerServer.getInstance();
            ApplicationContext app = broker.getApplication(username,projectId);
            Qos qos = null;

            if (app != null) {
                // update qos
                qos = app.getQos();
                qos.setBudget(Float.parseFloat(budget));
                qos.setDeadline(BiogridUtil.convertDeadline(deadline));
                qos.setOptimisation(ScheduleOptimisationType.intValue(schedAlgo));
                app.setQos(qos);
                broker.updateApplication(app);
            }
            
            // SET QoS IN DATABASE
            querier = new DBQuerier();
            querier.removeQoS(Integer.parseInt(projectId));
            querier.setQoS(Integer.parseInt(projectId),BiogridUtil.convertDeadline(deadline),budget,schedAlgo);

            if (request.getParameter("setAlgo")!=null && request.getParameter("setAlgo").compareTo("Set Algorithm")==0) {
                //Set Algo button clicked
                qosMsg = "Algorithm changed successfully.";
            }else if (request.getParameter("setDeadline")!=null && request.getParameter("setDeadline").compareTo("Set Deadline")==0) {
                //Set Deadline button clicked
                qosMsg = "Deadline changed successfully.";
            }else if (request.getParameter("setBudget")!=null && request.getParameter("setBudget").compareTo("Set Budget")==0) {
                //Set Budget button clicked
                qosMsg = "Budget changed successfully.";
            }else if (request.getParameter("setAll")!=null && request.getParameter("setAll").compareTo("Set QoS parameters")==0){
                qosMsg = "Algorithm changed successfully.";
                qosMsg += "<br />" + "Deadline changed successfully.";
                qosMsg += "<br />" + "Budget changed successfully.";
            }

            if (qos != null){
                session.setAttribute("qosSet","true",PortletSession.APPLICATION_SCOPE);
//            }else{
//                qosMsg = "The broker is not initialised. QoS parameters not set.";
//                session.setAttribute("qosSet","false",PortletSession.APPLICATION_SCOPE);
            }

            System.out.println("-------------------------- QoSMsg : "+qosMsg);
            
        } catch (Exception e) {
            qosMsg = "Setting QoS failed "+e.getMessage();
            e.printStackTrace();
        }
        
        response.setRenderParameter("qosMsg",qosMsg);
    }
    
//    private String setAlgo(GridbusFarmingEngine fe,  String schedAlgo){
//    	try{
//    	    if (fe==null){
//    	        return "Farming engine is null.";
//    	    }
//    	    Scheduler sch = fe.getScheduler();
//    	    // no scheduler exists so create a new one and set the algorithm.
//    	    if (sch==null){
//    	        fe.setScheduler(SchedulerFactory.getScheduler(schedAlgo,fe));
//			    return "Algorithm changed successfully.";
//    	    }
//			if (!fe.isSchedulingComplete()){
//				//it has either not started, or it has completed.
//				//we set it here assuming it has not started yet :)
//			    
//			    if (sch!=null){
//			       sch.setAlgorithm(schedAlgo); 
//			    }
//			    return "Algorithm changed successfully.";
//			}else{
//				//we can change algo only for the DBScheduler, 
//				//if it has started
//				if (sch instanceof DBScheduler){
//				    sch.setAlgorithm(schedAlgo); 
//					return "Algorithm changed successfully.";
//				}else{
//			        return "The scheduling algorithm can be changed for non-data apps only.";
//				}
//			}
//    	}catch(Exception e){
//    		return "Couldnot set algorithm: "+e.getMessage();
//    	}
//    }
//    
//    private String setDeadline(GridbusFarmingEngine fe,String deadline){
//    	try{
//			//Set Deadline button clicked
//			fe.setDeadline(deadline);
//			return "Deadline changed successfully.";
//    	}catch(Exception e){
//    		return "Couldnot set deadline: "+e.getMessage();
//    	}
//    }
//
//    private String setBudget(GridbusFarmingEngine fe,String budget){
//		try{
//			//Set Budget button clicked
//			fe.setBudget(Float.parseFloat(budget));
//			return "Budget changed successfully.";
//		}catch (Exception e){
//			return "Couldnot set budget: "+e.getMessage();
//		}
//    }

    /**
     * Set up any required view parameters.
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

        long deadline = -1;
        String budget = "";
        String optimisation = "";
        String HH = "";
        String MM = "";
        String SS = "";
        String DD = "";
        String MMM = "";
        String YYYY = "";
        String qosMsg = "";
        
    	// get proxy
        PortletSession session = request.getPortletSession(true);
       	// get project id
       	String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);

        if (projectId != null) {
       	
           	try {        
    	        DBQuerier querier = new DBQuerier();
                
                optimisation = querier.getOptimisation(Integer.parseInt(projectId));
    	        deadline = querier.getDeadline(Integer.parseInt(projectId));
    
    	        //set defaults, in case nothing is selected
    	        budget = querier.getBudget(Integer.parseInt(projectId));
    	        if (budget==null || budget.trim().equals("")) {
    	            budget = "100";
    	        }
    
    	        GregorianCalendar gcal = new GregorianCalendar(Locale.ENGLISH);
    	        if (deadline > 0) {
    	            gcal.setTimeInMillis(deadline);
    	        } else {
    	            gcal.setTimeInMillis(System.currentTimeMillis());
    	        }
    	
    	    	HH = Integer.toString(gcal.get(GregorianCalendar.HOUR_OF_DAY));
    	    	MM = Integer.toString(gcal.get(GregorianCalendar.MINUTE));
    	    	SS = Integer.toString(gcal.get(GregorianCalendar.SECOND));
    	    	if (deadline > 0) {
    	    	    DD = Integer.toString(gcal.get(GregorianCalendar.DAY_OF_MONTH));
    	    	} else {
    	    	    DD = Integer.toString(gcal.get(GregorianCalendar.DAY_OF_MONTH)+1);
    	    	}
    	    	switch (gcal.get(GregorianCalendar.MONTH)){
    	    	case GregorianCalendar.JANUARY:
    	    		MMM = "Jan";
    	    		break;
    	    	case GregorianCalendar.FEBRUARY:
    	    		MMM = "Feb";
    	    		break;
    	    	case GregorianCalendar.MARCH:
    	    		MMM = "Mar";
    	    		break;
    	    	case GregorianCalendar.APRIL:
    	    		MMM = "Apr";
    	    		break;
    	    	case GregorianCalendar.MAY:
    	    		MMM = "May";
    	    		break;
    	    	case GregorianCalendar.JUNE:
    	    		MMM = "Jun";
    	    		break;        		
    	    	case GregorianCalendar.JULY:
    	    		MMM = "Jul";
    	    		break;
    	    	case GregorianCalendar.AUGUST:
    	    		MMM = "Aug";
    	    		break;
    	    	case GregorianCalendar.SEPTEMBER:
    	    		MMM = "Sep";
    	    		break;
    	    	case GregorianCalendar.OCTOBER:
    	    		MMM = "Oct";
    	    		break;
    	    	case GregorianCalendar.NOVEMBER:
    	    		MMM = "Nov";
    	    		break;
    	    	case GregorianCalendar.DECEMBER:
    	    		MMM = "Dec";
    	    		break;
    	    	}
    	    	YYYY = Integer.toString(gcal.get(GregorianCalendar.YEAR));        
    
                if (querier.QoSexists(Integer.parseInt(projectId))) {
        	        qosMsg = request.getParameter("qosMsg");
                }
           	} catch (Exception e) { e.printStackTrace(); }

           	request.setAttribute("url", response.createActionURL().toString());
            request.setAttribute("schedAlgo", optimisation);
            request.setAttribute("budget", budget);
            request.setAttribute("HH", HH);
            request.setAttribute("MM", MM);
            request.setAttribute("SS", SS);
            request.setAttribute("DD", DD);
            request.setAttribute("MMM", MMM);
            request.setAttribute("YYYY", YYYY);
    
           	request.setAttribute("qosMsg",qosMsg);

           	getPortletContext().getRequestDispatcher("/jsp/qos.jsp").include(request, response);

        } else {
            getPortletContext().getRequestDispatcher("/jsp/selectprojecterror.jsp").include(request, response);
        }

    }
}
