/*
 * Title            :  GBBResults.java
 * Package      	:  examples
 * Project        	:  Gridbus Broker
 * Description   	:		
 * Created on   	:  March 31, 2005
 * Author	     	:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright     	:   (c) 2005, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */
package org.gridbus.biogrid.portlets.gridbus;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.server.BrokerServer;
import org.gridbus.broker.util.BrokerUtil;

/**
 * This portlet is used to copy and display the output files of the Gridbus Broker.
 *
 */
public class GBBResults extends GenericPortlet {
	
	/**
	 * Nothing much to do here.
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        
        // get proxy
        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String username = request.getRemoteUser();
        
        // get parameters from the form
        String filename = request.getParameter("filename");
        String brokerpath = "";
        String path = request.getParameter("path");
        if (path == null) path = "";
        
        try {
            // get broker
            BrokerServer broker = BrokerServer.getInstance();
            ApplicationContext app = broker.getApplication(username, projectId);
            brokerpath = app.getLocalOutputDirectory();
            
        } catch (Exception e) {
            e.printStackTrace();
        }

        File file = new File(BrokerUtil.combinePath(new String[] {brokerpath, path, filename}));

        System.out.println("---------- current file is = "+file.getAbsolutePath());
        
        if (filename.equals("..")) {
            // remove one level of directory
            int index = path.lastIndexOf('/');
            if (index>=0) {
                path = path.substring(0,index);
            }
            
        } else if (file.isDirectory()) {
            path = BrokerUtil.combinePath(path,filename);
        } else {
//            request.setAttribute("path", path);
            // download file here.
        }
        
        response.setRenderParameter("path", path);
        
        System.out.println("---------- current path is = "+path);
    }

	/**
	 * The broker puts its output files in a temporary directory.  Everytime the portlet
	 * refreshes itself, we check the temporary directory for outputs, if any exist we move them
	 * to a directory within our webapp so we can then access and display them.
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {

    	// get proxy
        PortletSession session = request.getPortletSession(true);
        String projectId = (String) session.getAttribute("projectId", PortletSession.APPLICATION_SCOPE);
        String username = request.getRemoteUser();
        
        String brokerpath = "";
        String path = request.getParameter("path");
        if (path == null) path = "";
        
        System.out.println("---------- current path is (view) = "+path);
        
        Vector results_files = new Vector();
        Vector results_dirs = new Vector();
        
        System.out.println("-------- viewing outputs ...");
        
    	try {
            // get broker
            BrokerServer broker = BrokerServer.getInstance();
            ApplicationContext app = broker.getApplication(username, projectId);
            
	        if (app != null) {
                
                System.out.println("-------- appID = "+app.getId());
                
	            // list files in the current directory
                brokerpath = app.getLocalOutputDirectory();
                
                File tempDir = new File(BrokerUtil.combinePath(brokerpath,path));
	        	// display results
                
                System.out.println("-------- location of dir is = "+tempDir.getAbsolutePath());
                
                File[] files = tempDir.listFiles();
                for (int i=0;i<files.length;i++) {
                    if (files[i].isDirectory()) {
                        results_dirs.addElement(files[i].getName());
                    } else if (files[i].isFile()) {
                        results_files.addElement(files[i].getName());
                    }
                }
	        }
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
        request.setAttribute("results_files", results_files);
        request.setAttribute("results_dirs", results_dirs);
        request.setAttribute("user", username);
        
        request.setAttribute("projectId", projectId);
        request.setAttribute("url", response.createActionURL().toString());
        request.setAttribute("path",path);

        getPortletContext().getRequestDispatcher("/jsp/results.jsp").include(request, response);
    }
}