/*
 * Title        :  BiogridUtil.java
 * Package      :  org.gridbus.biogrid.pdb
 * Project      :  BioGrid
 * Description	:  		
 * Created on   :  Apr 22, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2004, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.biogrid.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import javax.portlet.PortletRequest;

import org.apache.xmlbeans.XmlOptions;
import org.gridbus.biogrid.pdb.ProjectFile;
import org.gridbus.broker.common.Job;
import org.gridbus.broker.common.WorkUnit;
import org.gridbus.broker.constants.JobType;
import org.gridbus.broker.constants.LocationType;
import org.gridbus.broker.jobdescription.Task;
import org.gridbus.broker.jobdescription.commands.CopyCommand;
import org.gridbus.broker.jobdescription.commands.ExecuteCommand;
import org.gridbus.broker.util.BrokerUtil;
import org.gridbus.schemas.xpml.x2006.x01.xpml.CopyCommandType;
import org.gridbus.schemas.xpml.x2006.x01.xpml.ExecuteCommandType;
import org.gridbus.schemas.xpml.x2006.x01.xpml.XpmlDocument;
import org.gridbus.schemas.xpml.x2006.x01.xpml.CopyCommandType.Destination;
import org.gridbus.schemas.xpml.x2006.x01.xpml.CopyCommandType.Source;
import org.gridbus.schemas.xpml.x2006.x01.xpml.XpmlDocument.Xpml;
import org.gridlab.gridsphere.portlet.User;
import org.gridlab.gridsphere.portlet.service.spi.PortletServiceFactory;
import org.gridlab.gridsphere.portlet.service.spi.impl.SportletServiceFactory;
import org.gridlab.gridsphere.services.core.user.LoginUserModule;
import org.gridlab.gridsphere.services.core.user.impl.LoginServiceImpl;
import org.gridlab.gridsphere.services.security.gss.CredentialManagerService;
import org.ietf.jgss.GSSCredential;
import org.w3c.dom.Node;

/**
 * @author hag
 * 
 * Provides some helper methods mostly useful for BioGrid.
 * 
 */
public class BiogridUtil {

    /**
     * @param deadline
     * @return
     */
    public static long convertDeadline(String deadline) {
        long d;
		Date nxtd=new Date();
		Locale loc=Locale.ENGLISH;
		DateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy hh:mm:ss",loc);
		try{
			nxtd = dateFormat.parse(deadline); //just to check if it is in the correct format
			d = nxtd.getTime();
		} catch (ParseException e){
			d = System.currentTimeMillis();
		}
		return nxtd.getTime();
    }

    /**
     * @param deadline
     * @return
     */
    public static String convertDeadline(long deadline) {
        String d;
		Date nxtd=new Date(deadline);
		Locale loc=Locale.ENGLISH;
		DateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy hh:mm:ss",loc);
		d = dateFormat.format(nxtd); //just to check if it is in the correct format
		return d;
    }

	/**
	 * Converts a date into a specified format according to SimpleDateFormat.
	 * default is in form: EEE, d MMM yyyy HH:mm:ss Z
	 * 
	 * @param dateMillis (in milliseconds)
	 * @param pattern
	 * @return
	 */
	public static String formatDate(long dateMillis, String pattern){
	    
	    Date date = new Date (dateMillis);
	    SimpleDateFormat sdf = new SimpleDateFormat();
	    if (pattern != null) {
	        sdf.applyPattern(pattern);
	    } else {
	        sdf.applyPattern("EEE, d MMM yyyy HH:mm:ss Z");
	    }
		return sdf.format(date);
	}

	/**
	 * @param data
	 * @param path
	 * @return
	 * @throws IOException
	 */
	public static File writeFile(String data, String path) throws IOException {
	    if (data != null) {
	        return writeFile(data.getBytes(),path);
	    } else {
	        return null;
	    }
	}

	/**
	 * @param data
	 * @param path
	 * @return
	 * @throws IOException
	 */
	public static File writeFile(byte[] data, String path) throws IOException {
		File f = new File(path);
		f.createNewFile();
		if (f.canWrite()) {
			FileOutputStream out = new FileOutputStream(f);
			out.write(data);
			out.flush();
			out.close();
		}
		return f;
	}
	
    
	/**
     * @param data
     * @return table
     */
    public static Hashtable getAsTable(String data) {
        Hashtable table = new Hashtable();
        try {
            String line = "";
            String key = "";
            String value = "";
            StringTokenizer strTok = null;
            BufferedReader br = new BufferedReader(new StringReader(data));
            for (line = br.readLine();line!=null;line = br.readLine()) {
                strTok = new StringTokenizer(line);
                if (strTok.countTokens() == 2) {
                    key = strTok.nextToken();
                    value = strTok.nextToken();
                    table.put(key.trim(),value.trim());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return table;
    }
    
	/**
	 * Workaround to get the Gridsphere User object without having access to 
	 * the Gridsphere specific request/response 
	 * 
	 * @param request
	 * @return
	 */
	public static User getUser(PortletRequest request) {
    	Map userInfo = (Map)request.getAttribute(PortletRequest.USER_INFO);
        String username = (String)userInfo.get("user.name");
		return getUser(username);
	}
    
    /**
     * Workaround to get the Gridsphere User object without having access to 
     * the Gridsphere specific request/response 
     * @param username 
     * @return
     */
    public static User getUser(String username) {
        LoginServiceImpl service = new LoginServiceImpl();
        LoginUserModule activeLoginModule = service.getActiveLoginModule();
        User user = activeLoginModule.getLoggedInUser(username);
        return user;
    }
	
    public static GSSCredential getUserProxy(User user) {
        GSSCredential proxy = null;
        try {
            PortletServiceFactory factory = SportletServiceFactory.getInstance();
            CredentialManagerService credentialManagerService = 
                (CredentialManagerService) factory.createPortletService(CredentialManagerService.class, null, true);
            proxy = credentialManagerService.getDefaultCredential(user);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return proxy;
    }
    
    public static WorkUnit createWorkUnit(int projectId, String command, String[] input, String[] output, String outputURLWithPath) {

        Job job = new Job();
        Task task = new Task();
        
        try {

            String filename = null;
            String url = null;
            
            // COPY ALL INPUT FILES //
            for (int i=0; i < input.length; i++) {
                url = input[i];
                if (!"".equals(url)) {
                    CopyCommand cc = new CopyCommand();
                    cc.setSource(LocationType.REMOTE, url);
                    String[] parts = url.split("/");
                    cc.setDestination(LocationType.NODE, parts[parts.length-1]);
                    task.addCommand(cc);
                }
            }
            
            // EXECUTE COMMAND //
            ExecuteCommand ec = new ExecuteCommand();
            ec.setExecutable(command);
            task.addCommand(ec);
            
            // COPY OUT ALL REQUIRED OUTPUTS //
            for (int i = 0; i< output.length; i++) {
                filename = output[i];
                if (filename!=null && !"".equals(filename.trim())) {
                    CopyCommand cc = new CopyCommand();
                    cc.setSource(LocationType.NODE, filename);
                    cc.setDestination(LocationType.REMOTE, BrokerUtil.combinePath(outputURLWithPath,filename));
                    task.addCommand(cc);
                }
            }
            
        } catch (Exception e) {
            e.printStackTrace();
        }

        job.setTask(task);
        job.setName("job1");
        job.setType(JobType.USER);

        return job;
    }
    
    
    public static ProjectFile generateApplication(int projectId, String command, String[] input, String[] output, String outputURLWithPath) {
        
        ProjectFile application = new ProjectFile();
        
        Node node = null;

        try {

            String filename = null;
            String url = null;

            XpmlDocument doc = XpmlDocument.Factory.newInstance();
            Xpml xpml = doc.addNewXpml();

            // TASK //
            Xpml.Task task = xpml.addNewTask();
            
            // COPY ALL INPUT FILES //
            for (int i=0; i < input.length; i++) {
                url = input[i];
                if (!"".equals(url)) {
                    CopyCommandType copyi = task.addNewCopy();
                    Source copyi_src = copyi.addNewSource();
                    copyi_src.setLocation(CopyCommandType.Source.Location.REMOTE);
                    copyi_src.setFile(url);
                    Destination copyi_dest = copyi.addNewDestination();
                    copyi_dest.setLocation(CopyCommandType.Destination.Location.NODE);
                    String[] parts = url.split("/");
                    copyi_dest.setFile(parts[parts.length-1]);
                }
            }
            
            // EXECUTE COMMAND //
            ExecuteCommandType ec = task.addNewExecute();
            ec.addNewCommand().setValue(command);

            // COPY OUT ALL REQUIRED OUTPUTS //
            for (int j = 0; j< output.length; j++) {
                filename = output[j];
                if (filename!=null && !"".equals(filename.trim())) {
                    CopyCommandType copyo = task.addNewCopy();
                    Source copyo_src = copyo.addNewSource();
                    copyo_src.setLocation(CopyCommandType.Source.Location.NODE);
                    copyo_src.setFile(filename);
                    Destination copyo_dest = copyo.addNewDestination();
                    copyo_dest.setLocation(CopyCommandType.Destination.Location.REMOTE);
                    copyo_dest.setFile(BrokerUtil.combinePath(outputURLWithPath,filename));
                    System.out.println("-------------- adding file : "+filename);
                }
            }
            
            node = doc.getDomNode();

            XmlOptions opt = new XmlOptions();
            opt.setSavePrettyPrint();
            application.setData(doc.xmlText(opt).getBytes());

        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return application;
    }
    
    /**
     * @param contactString
     * @return
     */
    public static String getJobManager(String contactString){
        //We assume a server is defined by the following string:
        //  <name>:<port>/<jobmanager>
        //   
        String[] jobManagerArr=Pattern.compile("/").split(contactString,0);
        String jobManager = "";
        if (jobManagerArr.length > 1) {
            jobManager=jobManagerArr[1];
        }
        return jobManager;
    }
    
    /**
     * @param file
     * @return
     */
    public static String readFile(File file) {
        String data = null;
        try {
            FileInputStream fis = new FileInputStream(file);
            BufferedReader br = new BufferedReader(new InputStreamReader(fis));
            String line = "";
            data = "";
            while ((line = br.readLine()) != null) {
                data += line+"\n";
            }
        } catch (FileNotFoundException e) {
            data = "Error reading file.";
        } catch (IOException e2) {
            data = "Error reading file.";
        }
        return data;
    }
}
