/*
 * Title : DownloadServlet.java Package : org.gridbus.biogrid.util Project :
 * BioGrid Description : Created on : May 17, 2005 Author : Hussein Gibbins
 * (hag@cs.mu.oz.au) Copyright : (c) 2004, Grid Computing and Distributed
 * Systems Laboratory, Dept. of Computer Science and Software Engineering,
 * University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. See the GNU General Public License
 * (http://www.gnu.org/copyleft/gpl.html)for more details.
 *  
 */
package org.gridbus.biogrid.util;

import java.io.IOException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.gridbus.biogrid.chart.Chart;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.common.Broker;
import org.gridbus.broker.common.ComputeServer;
import org.gridbus.broker.common.GridbusFarmingEngine;
import org.gridbus.broker.common.JobStatistics;
import org.gridbus.broker.common.Qos;
import org.gridbus.broker.constants.JobStatus;
import org.gridbus.broker.constants.ServiceType;
import org.gridbus.broker.exceptions.GridBrokerException;
import org.gridbus.broker.gui.model.XYData;
import org.gridbus.broker.gui.view.BrokerConsole;
import org.gridbus.broker.server.BrokerServer;


/**
 * @author hag
 *
 * Simple download servlet used to provide ability to download files from webapp. Requires
 * servlet mapping in web.xml and servlet-api.jar
 * 
 */
public final class BrokerChartFactoryServlet extends HttpServlet {

    // problem with every request obtaining a different session. so need to store a thread for each app.
    private static Hashtable apps = new Hashtable();
    private static Hashtable threads = new Hashtable(); // ThreadGroup?
    
    public final static int TYPE_JOB_COMPLETION = 0;
    public final static int TYPE_JOB_STATUS = 1;
    public final static int TYPE_JOBS_ON_SERVER = 2;
    public final static int CHART_TIME_RANGE = 12; // 12 points on the graph
    public final static int CHART_TIME_INTERVAL = 5000; // collect data every 5 seconds
    
    private synchronized void initApplication(final String appId) {
        System.out.println("------------------------------------------------ Initialising Application !! "+Thread.currentThread().getName());
        // check if app already exists. if not add it so that the thread can update stats for it.
        if (!apps.containsKey(appId)) {
            // first time accessing for this application. so do some init!
            apps.put(appId, new Hashtable());
            try {
                Thread updateThread = new Thread(new Runnable(){
                    public void run(){
                        while (true) {
                            try {
                                updateData(appId);
                            } catch (Exception e) { }
                            try {
                                Thread.sleep(CHART_TIME_INTERVAL);
                            } catch (Exception e) { }
                        }
                    }
                });
                updateThread.start();
                threads.put(appId, updateThread);
            } catch (Exception e) {
                apps.remove(appId);
                threads.remove(appId);
            }
        }
    }
    
    private void updateData(String appId) throws Exception {
        
//        System.out.println("------------------------------------------------ UPDATING DATA FOR ("+appId+")!! "+Thread.currentThread().getName());
/*        
        // for each app update Jobs, Servers, QoS data
        for (Enumeration en = apps.keys(); en.hasMoreElements();) {
            String appId = (String) en.nextElement();
*/
            // get broker app context
            BrokerServer broker = BrokerServer.getInstance();
            // update job data
            updateJobData((Hashtable) apps.get(appId), broker, appId);
            updateServerData((Hashtable) apps.get(appId), broker, appId);
            updateQoSData((Hashtable) apps.get(appId), broker, appId);
            
//            printStats((Hashtable) apps.get(appId));
/*            
        }
*/
    }
    
    private void printStats(Hashtable data) {
        
        System.out.println("------------------------------------------------ PRINTING STATS !! ");
        
        // print out what is there for testing purposes...
        System.out.println("*********************************************** Budget "+data.get("Budget"));
        System.out.println("*********************************************** Budget Spent "+data.get("Budget Spent"));
        System.out.println("*********************************************** Deadline "+data.get("Deadline"));
        System.out.println("*********************************************** Time Spent "+data.get("Time Spent"));
        
        Hashtable test = (Hashtable) data.get("Server Done Jobs");
        for (Enumeration en=test.keys();en.hasMoreElements();) {
            String hn = (String) en.nextElement();
            LinkedList list = (LinkedList) test.get(hn);
            // print for testing
            System.out.println("*********************************************** x Done Jobs for "+hn+" "+list.getLast());    
        }

        // print out what is there for testing purposes...
        System.out.println("*********************************************** Done Jobs "+data.get("Done Jobs"));
        System.out.println("*********************************************** Failed Jobs "+data.get("Failed Jobs"));
        System.out.println("*********************************************** Scheduled Jobs "+data.get("Scheduled Jobs"));
        System.out.println("*********************************************** Stagein Jobs "+data.get("Stagein Jobs"));
        System.out.println("*********************************************** Stageout Jobs "+data.get("Stageout Jobs"));
        System.out.println("*********************************************** Ready Jobs "+data.get("Ready Jobs"));
        System.out.println("*********************************************** Running Jobs "+data.get("Running Jobs"));
        
    }
    
    private void updateServerData(Hashtable data, Broker broker, String appId) {
        try {
            if (!data.containsKey("Server Done Jobs")) {
                data.put("Server Done Jobs", new Hashtable());
            }
            Hashtable serverData = (Hashtable) data.get("Server Done Jobs");

            Collection servers = broker.getServices(appId, ServiceType.COMPUTE);
            if(servers != null){
                Object[] serverArray = servers.toArray();
                for(int i=0;i<serverArray.length;i++){
                    ComputeServer each =(ComputeServer) serverArray[i];
                    String hostname = each.getHostname();
                    JobStatistics jobStats = broker.getJobStatistics(appId,each.getName());
                    if (jobStats == null)
                        jobStats = new JobStatistics(); //just to be safe.
                    
                    addData(serverData,hostname,""+jobStats.getTotalJobs());
                }
            }
            
        } catch (Exception e) {
        }
    }

    private void updateJobData(Hashtable data, Broker broker, String appId) {
        try {
            // get each linked list from the hashtable under each job type
            int doneJobs = broker.getJobs(appId, JobStatus.DONE).size();
            int failedJobs = broker.getJobs(appId, JobStatus.FAILED).size();
            int scheduledJobs = broker.getJobs(appId, JobStatus.SCHEDULED).size(); 
            int stageInJobs = broker.getJobs(appId, JobStatus.STAGE_IN).size();
            int stageOutJobs =broker.getJobs(appId, JobStatus.STAGE_OUT).size();
            int readyJobs = broker.getJobs(appId, JobStatus.READY).size();
            int runningJobs = broker.getJobs(appId, JobStatus.ACTIVE).size()+broker.getJobs(appId, JobStatus.SUBMITTED).size()
                +stageInJobs+stageOutJobs;
        
            addData(data,"Done Jobs", ""+doneJobs);
            addData(data,"Failed Jobs", ""+failedJobs);
            addData(data,"Scheduled Jobs", ""+scheduledJobs);
            addData(data,"Stagein Jobs", ""+stageInJobs);
            addData(data,"Stageout Jobs", ""+stageOutJobs);
            addData(data,"Ready Jobs", ""+readyJobs);
            addData(data,"Running Jobs", ""+runningJobs);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateQoSData(Hashtable data, Broker broker, String appId) {
        try {
            Map qosData  = new HashMap();
            ApplicationContext ctx = broker.getApplication(appId);
            Qos qos = ctx.getQos();
            qosData.put("cost",new Double(qos.getBudgetSpent()));
            qosData.put("time",new Double(qos.getTimeSpent()));
            
            int doneJobs = broker.getJobs(appId, JobStatus.DONE).size();
            int totalJobs = broker.getJobs(appId, JobStatus.STATUS_ANY).size();
            
            if(totalJobs != 0)
                data.put("doneRate", ""+((double)doneJobs/(double)totalJobs));
            else
                data.put("doneRate", "0");
    
            addData(data, "Budget", ""+qos.getBudget());
            addData(data, "Budget Spent", ""+qos.getBudgetSpent());
            addData(data, "Deadline", ""+qos.getDeadline());
            addData(data, "Time Spent", ""+qos.getTimeSpent());

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    
    
    private void addData(Hashtable data, String series, String value) {

        LinkedList values = (LinkedList) data.get(series);
        if (values == null) {
            values = new LinkedList();
            data.put(series, values);
        }
        
        values.addFirst(value);
        
        if (values.size() > CHART_TIME_RANGE) {
            values.removeLast();
        }
    }
    
    /**
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */
	public void service(HttpServletRequest request, HttpServletResponse response)
	throws ServletException, java.io.IOException {

        String appId = request.getParameter("appId");
	    String chartType = request.getParameter("type");
        String updateURL = "../gridbusportlets/chart?appId="+appId+"&type="+chartType;

        if (appId != null && chartType != null) {
            
            initApplication(appId);

            try {
                int type = Integer.parseInt(chartType);
                
                Chart chart = makeChart((Hashtable) apps.get(appId), type);
                if (updateURL != null && !"".equals(updateURL)) {
                    chart.setLiveUpdateURL(updateURL);
                }
                
                System.out.println("------- chart: \n"+chart.toString());
                response.getWriter().println(chart);
                response.getWriter().flush();
                
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
	}
    
    private Chart makeChart(Hashtable data, int type) throws Exception {
        
        Chart chart = new Chart();
        
        switch (type) {
        case TYPE_JOB_COMPLETION:
            chart = makeJobCompletionChart(data);
            break;
        case TYPE_JOB_STATUS:
            chart = makeJobStatusChart(data);
            break;
        case TYPE_JOBS_ON_SERVER:
            chart = makeJobsOnServerChart(data);
        default:
            break;
        }
        
        return chart;
    }
    
    /**
     */
    private static Chart makeJobCompletionChart(Hashtable data) throws GridBrokerException {
        
        Chart chart = new Chart();
        String title = "Job Completion";

        Hashtable servers = (Hashtable) data.get("Server Done Jobs");
        
        if (servers != null) {
            for (Enumeration en=servers.keys();en.hasMoreElements();) {
                String hostname = (String) en.nextElement();
                LinkedList values = (LinkedList) servers.get(hostname);
                
                chart.addColumnLabel(hostname);
                chart.addRowData(title, ""+values.getFirst());
            }
        }
        
        chart.setExtra("<chart_transition type='dissolve' delay='0.1' duration='0.3' order='category' />");
        chart.addChartType("3d pie");
        return chart;
    }
    
    private static Chart makeJobStatusChart(Hashtable data) throws GridBrokerException {

        Chart chart = new Chart();
        String title = "Job Statuses";
        
        if (data.get("Done Jobs") != null) {

            System.out.println("_+_+_+_+_+_+_+ Object type is : "+data.get("Done Jobs").getClass().getName()+ " expected (String)");
            
            String doneJobs = (String) ((LinkedList) data.get("Done Jobs")).getFirst();
            String failedJobs = (String) ((LinkedList) data.get("Failed Jobs")).getFirst();
            String scheduledJobs = (String) ((LinkedList) data.get("Scheduled Jobs")).getFirst(); 
            String stageInJobs = (String) ((LinkedList) data.get("Stagein Jobs")).getFirst();
            String stageOutJobs = (String) ((LinkedList) data.get("Stageout Jobs")).getFirst();
            String readyJobs = (String) ((LinkedList) data.get("Ready Jobs")).getFirst();
            String activeJobs = (String) ((LinkedList) data.get("Running Jobs")).getFirst();
    
            String[] labels = new String[] {JobStatus.stringValue(JobStatus.DONE),
                                            JobStatus.stringValue(JobStatus.FAILED),
                                            JobStatus.stringValue(JobStatus.STAGE_IN),
                                            JobStatus.stringValue(JobStatus.STAGE_OUT),
                                            JobStatus.stringValue(JobStatus.SCHEDULED),
                                            JobStatus.stringValue(JobStatus.READY),
                                            "Running"};
    
            String[] values = new String[] {doneJobs, 
                                            failedJobs, 
                                            stageInJobs, 
                                            stageOutJobs, 
                                            scheduledJobs, 
                                            readyJobs, 
                                            activeJobs};
    
            chart.setColumnLabels(labels);
            chart.addRowData(title, doneJobs);
            chart.addRowData(title, failedJobs);
            chart.addRowData(title, stageInJobs);
            chart.addRowData(title, stageOutJobs);
            chart.addRowData(title, scheduledJobs);
            chart.addRowData(title, readyJobs);
            chart.addRowData(title, activeJobs);
        }
        
        chart.addChartType("bar");

        return chart;
    }
    
    private static Chart makeJobsOnServerChart(Hashtable data) {
        
        Chart chart = new Chart();
        String title = "Jobs on Server";

        Hashtable servers = (Hashtable) data.get("Server Done Jobs");
        
        if (servers != null) {
            for (Enumeration en=servers.keys();en.hasMoreElements();) {
                String hostname = (String) en.nextElement();
                LinkedList values = (LinkedList) servers.get(hostname);
                for (int i=0;i<values.size();i++) {
                    chart.addRowData(hostname, ""+values.get(i));
                }
            }
            
            for (int i=0;i<CHART_TIME_RANGE;i++) {
                chart.addColumnLabel(""+((CHART_TIME_INTERVAL * i)/1000));
            }
        }
        
        chart.addChartType("line");
        return chart;
    }
}
