/*
 * Title : DownloadServlet.java Package : org.gridbus.biogrid.util Project :
 * BioGrid Description : Created on : May 17, 2005 Author : Hussein Gibbins
 * (hag@cs.mu.oz.au) Copyright : (c) 2004, Grid Computing and Distributed
 * Systems Laboratory, Dept. of Computer Science and Software Engineering,
 * University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version. See the GNU General Public License
 * (http://www.gnu.org/copyleft/gpl.html)for more details.
 *  
 */
package org.gridbus.biogrid.util;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.gridbus.biogrid.data.DBQuerier;
import org.gridbus.biogrid.data.DataQuerier;
import org.gridbus.biogrid.pdb.DataResource;
import org.gridbus.broker.common.ApplicationContext;
import org.gridbus.broker.server.BrokerServer;
import org.gridbus.broker.util.BrokerUtil;
import org.gridlab.gridsphere.portlet.User;

/**
 * @author hag
 *
 * Simple download servlet used to provide ability to download files from webapp. Requires
 * servlet mapping in web.xml and servlet-api.jar
 * 
 */
public final class DownloadServlet extends HttpServlet {
    /**
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */
	public void service(HttpServletRequest request, HttpServletResponse response)
	throws ServletException, java.io.IOException {

        String dataresourceId = request.getParameter("dataresourceId");
        String filePath = request.getParameter("filePath");
		String fileId = request.getParameter("fileId");
        String projectId = request.getParameter("projectId");
        String username = request.getParameter("user");

        User user = BiogridUtil.getUser(username);
        
        String file = "";
        
		if (fileId != null) {
			try {

                if (dataresourceId == null || "".equals(dataresourceId)) {
                    // get local file
                    file = getLocalFileData(filePath, fileId, username, projectId);
                } else {
                    // get remote file
                    file = getRemoteFileData(filePath, fileId, dataresourceId, user);
                }
                
                System.out.println("file : \n"+file);
                
				BufferedOutputStream out = new BufferedOutputStream(response.getOutputStream());
				String contentType = "application/unknown";
				response.setContentType(contentType);
				response.setHeader("Content-Disposition","attachment; filename=\"" + fileId + "\"");
				out.write(file.getBytes());
				out.flush();
				out.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
    
    private String getRemoteFileData(String filePath, String fileId, String dataresourceId, User user) throws Exception {
        DBQuerier querier = new DBQuerier();
        DataResource dr = querier.getDataResource(Integer.parseInt(dataresourceId));
        DataQuerier dq = new DataQuerier(dr,user);
        filePath = BrokerUtil.combinePath(dr.getHomedir(),filePath);
        
        System.out.println("dataresourceId : "+dataresourceId);
        System.out.println("filePath : "+filePath);
        System.out.println("fileId : "+fileId);
        System.out.println("username : "+user.getFullName());
        System.out.println("--------- current directory (download) : "+dq.getCurrentDirectory());
        
        return dq.getFile(filePath, fileId);
    }
    
    private String getLocalFileData(String filePath, String fileId, String username, String projectId) {
        String brokerpath = "";
        System.out.println("------- username : "+username);
        System.out.println("------- projectId : "+projectId);

        try {
            // get broker
            BrokerServer broker = BrokerServer.getInstance();
            ApplicationContext app = broker.getApplication(username, projectId);
            System.out.println("------- app id is : "+app.getId());
            brokerpath = app.getLocalOutputDirectory();
            System.out.println("------- dir is : "+brokerpath);
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        File file = new File(BrokerUtil.combinePath(new String[] {brokerpath, filePath, fileId}));
        System.out.println("--------- downloading local file : "+file.getAbsolutePath());
        
        return BiogridUtil.readFile(file);
    }
}
